/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.consumer;

import java.net.MalformedURLException;

import org.apache.log4j.Logger;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.query.AdhocQueryRequest;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.query.AdhocQueryResponse;
import org.openehealth.ipf.commons.ihe.xds.iti18.Iti18PortType;
import org.springframework.beans.factory.annotation.Autowired;

import dk.nsi.dds.projects.sxa.documentmetadataprovider.ws.MetadataProviderWS;

public class DSSDocumentQuerySender implements DocumentQuerySender {
	private static final Logger logger = Logger.getLogger(DSSDocumentQuerySender.class);

	@Autowired
	private Iti18PortType iti18PortType;

	public DSSDocumentQuerySender() {
	}

	public void initialize() throws MalformedURLException {
		if (this.iti18PortType == null) {
			logger.debug("Could not find autowired Iti18PortType - creating Iti18WS instance for this test");
			this.iti18PortType = new MetadataProviderWS();
		}
	}

	public AdhocQueryResponse sendQuery(AdhocQueryRequest request) throws Exception {
		initialize();
		return iti18PortType.documentRegistryRegistryStoredQuery(request);
	}

//	public void addServiceSpecificResults(AdhocQueryResponse response, PreliminaryDocumentMetadataQueryResult result) {
//		result.setFilteredByConsent(responseContainsConsent(response));
//	}
//
//	public void addServiceSpecificResults(AdhocQueryResponse response, PreliminarySimpleDocumentMetadataQueryResult result) {
//		result.setFilteredByConsent(responseContainsConsent(response));
//	}
//
//	private boolean responseContainsConsent(AdhocQueryResponse response) {
//		if (response.getRegistryErrorList() != null && response.getRegistryErrorList().getRegistryError() != null) {
//			for (RegistryError error : response.getRegistryErrorList().getRegistryError()) {
//				if (DSSConstants.DSS_CONSENT_FILTER_APPLIED_ERRORLIST_CODE_CONTEXT.equalsIgnoreCase(error.getCodeContext())) {
//					return true;
//				}
//			}
//		}
//		return false;
//	}
}