/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.consumer;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBElement;

import dk.nsi.dds.projects.sxa.common.AddTestSoapHeadersInterceptor;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.log4j.Logger;
import org.junit.Assert;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetRequestType;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetResponseType;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Identifiable;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocument;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.FindDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.GetDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.responses.RetrievedDocument;
import org.openehealth.ipf.commons.ihe.xds.core.responses.RetrievedDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.query.AdhocQueryRequest;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.query.AdhocQueryResponse;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.rim.ExternalIdentifierType;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.rim.ExtrinsicObjectType;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.rim.IdentifiableType;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.rim.InternationalStringType;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.rim.LocalizedStringType;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.rim.RegistryObjectListType;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.rim.SlotType1;
import org.openehealth.ipf.commons.ihe.xds.iti18.Iti18PortType;
import org.openehealth.ipf.commons.ihe.xds.iti43.Iti43PortType;
import org.openehealth.ipf.platform.camel.ihe.xds.core.converters.EbXML30Converters;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import dk.nsi.dds.projects.sxa.common.testutil.AssertUtil;
import dk.nsi.dds.projects.sxa.common.xds.impl.AdhocQueryRequestBuilder;

@Component
public class DocumentConsumerImpl implements DocumentConsumer {
	private static final Logger logger = Logger.getLogger(DocumentConsumerImpl.class);
	
	@Autowired
	private Iti18PortType iti18PortType;

	@Autowired
	private Iti43PortType iti43PortType;

	public DocumentConsumerImpl() {
	}

	public Iti18PortType getIti18Port() {
//		if (iti18PortType == null) {
//			logger.warn("Using dummy Iti18WS instance as autowiring failed.");
//			iti18PortType = new MetadataProviderWS();
//		}
		return iti18PortType;
	}

	public Iti43PortType getIti43Port() {
//		if (iti43PortType == null) {
//			logger.warn("Using dummy Iti43WS instance as autowiring failed.");
//			iti43PortType = new DocumentProviderWS();
//		}

		Client proxy = ClientProxy.getClient(iti43PortType);
		proxy.getOutInterceptors().add(new AddTestSoapHeadersInterceptor());

		return iti43PortType;
	}

	public FindDocumentsQuery createQuery(Identifiable patientId) {
		return createFindDocumentsQuery(patientId);
	}

	@Override
	public FindDocumentsQuery createFindDocumentsQuery(Identifiable patientId) {
		FindDocumentsQuery query = new FindDocumentsQuery();
		query.setPatientId(patientId);
		return query;
	}

	@Override
	public GetDocumentsQuery createGetDocumentsQuery() {
		return new GetDocumentsQuery();
	}

	public AdhocQueryResponse doQuery(FindDocumentsQuery query) throws Exception {
		AdhocQueryRequest request = new AdhocQueryRequestBuilder().createRegistryStoredQueryFindDocumentsRequest(query);
		AdhocQueryResponse response = getIti18Port().documentRegistryRegistryStoredQuery(request);
		return response;
	}

	public AdhocQueryResponse doQuery(GetDocumentsQuery query) throws Exception {
		AdhocQueryRequest request = new AdhocQueryRequestBuilder().createRegistryStoredQueryGetDocumentsRequest(query);
		AdhocQueryResponse response = getIti18Port().documentRegistryRegistryStoredQuery(request);
		return response;
	}
	
	@Override
	public List<RetrievedDocument> doRetrieve(List<RetrieveDocument> toRetrieve) throws Exception {
		return doRetrieveAll(toRetrieve).getDocuments();
	}
	
	@Override
	public RetrievedDocumentSet doRetrieveAll(List<RetrieveDocument> toRetrieve) throws Exception {
		RetrieveDocumentSet documentSet = new RetrieveDocumentSet();
		documentSet.getDocuments().addAll(toRetrieve);
		RetrieveDocumentSetRequestType documentSetRequestType = EbXML30Converters.convert(documentSet);
		RetrieveDocumentSetResponseType documentRepositoryRetrieveDocumentSet = getIti43Port().documentRepositoryRetrieveDocumentSet(documentSetRequestType);
		RetrievedDocumentSet retrievedDocumentSet = EbXML30Converters.convert(documentRepositoryRetrieveDocumentSet);
		return retrievedDocumentSet;
	}

	@Override
	public List<RetrieveDocument> prepareRetrieve(AdhocQueryResponse toRetrieve) {
		List<RetrieveDocument> result = new ArrayList<>();
		RegistryObjectListType registryObjectList = toRetrieve.getRegistryObjectList();
		List<JAXBElement<? extends IdentifiableType>> identifiables = registryObjectList.getIdentifiable();
		for (JAXBElement<? extends IdentifiableType> identifiable : identifiables) {
			AssertUtil.assertEquals(ExtrinsicObjectType.class, identifiable.getDeclaredType());
			ExtrinsicObjectType extrinsicObject = (ExtrinsicObjectType) identifiable.getValue();
	
			String repositoryUniqueId = null;
			List<SlotType1> slots = extrinsicObject.getSlot();
			for (SlotType1 slot : slots) {
				if("repositoryUniqueId".equals(slot.getName())) {
					repositoryUniqueId = slot.getValueList().getValue().get(0);
					break;
				}
			}
			
			String documentUniqueId = null;
			List<ExternalIdentifierType> externalIdentifiers = extrinsicObject.getExternalIdentifier();
			for (ExternalIdentifierType externalIdentifier : externalIdentifiers) {
				InternationalStringType name = externalIdentifier.getName();
				if(name != null && name.getLocalizedString() != null && name.getLocalizedString().size() == 1) {
					LocalizedStringType localizedStringType = name.getLocalizedString().get(0);
					if("XDSDocumentEntry.uniqueId".equals(localizedStringType.getValue())) {
						documentUniqueId = externalIdentifier.getValue();
					}
					
				}
			}
			
			String homeCommunityId = extrinsicObject.getHome();
			
			Assert.assertNotNull(homeCommunityId);
			Assert.assertNotNull(repositoryUniqueId);
			Assert.assertNotNull(documentUniqueId);

			RetrieveDocument doc = new RetrieveDocument();
			doc.setDocumentUniqueId(documentUniqueId);
			doc.setHomeCommunityId(homeCommunityId);
			doc.setRepositoryUniqueId(repositoryUniqueId);

			result.add(doc);
		}
		return result;
	}

//	public List<DocumentReference> prepareRetrieve(AdhocQueryResponse toRetrieve) {
//		return new DocumentsToRetrieveImpl(toRetrieve);
//	}
//
//	public List<RetrievedDocument> doRetrieve(DocumentReference toRetrieve) throws Exception {
//		if (toRetrieve == null || toRetrieve.getDocumentMetadataList().isEmpty()) {
//			throw new IllegalArgumentException("metadata for documents to retrieve missing");
//		}
//		if (toRetrieve.getDocumentMetadataList().size() > 1) {
//			logger.debug("Attempting retrieve of multiple documents by single call to service");
//		}
//		List<RetrievedDocument> result = doRetrieveBySingleInvocation(toRetrieve);
//		// If failing to retrieve all documents, this could be due to inability
//		// server-side to
//		// handle multiple documents in one invocation
//		if (toRetrieve.getDocumentMetadataList().size() > 1 && result != null && result.isStatusFailed() && result.getUnreachableDocuments() != null
//				&& result.getUnreachableDocuments().size() == toRetrieve.getDocumentMetadataList().size()) {
//			logger.debug("Attempting retrieve of documents by multiple calls to service");
//			result = doRetrieveByMultipleInvocations(toRetrieve);
//		}
//		return result;
//	}

//	private List<RetrievedDocument> doRetrieveBySingleInvocation(List<DocumentReference> toRetrieve) throws Exception {
//		RetrieveDocumentSetRequestType request = xdsTransformer.transform(toRetrieve);
//		RetrieveDocumentSetResponseType response = iti43PortType.sendRetrieve(request, toRetrieve.getPatientId());
//		PreliminaryRetrievedDocumentsResult result = xdsTransformer.transform(response, request, toRetrieve);
//		iti43PortType.addServiceSpecificResults(response, result);
//		return result;
//	}
//
//	private List<RetrievedDocument> doRetrieveByMultipleInvocations(List<DocumentReference> toRetrieve) throws Exception {
//		// Whilst awaiting upgrade of JBoss that enables multi-document retrieve, split
//		// retrieve into separate calls
//		// and merge results into one.
//		PreliminaryRetrievedDocumentsResult result = new RetrievedDocumentsResultImpl();
//		RetrieveDocumentSetRequestType originalRequest = xdsTransformer.transform(toRetrieve);
//		ObjectFactory objectFactory = new ObjectFactory();
//		for (DocumentRequest entry : originalRequest.getDocumentRequest()) {
//			RetrieveDocumentSetRequestType singleRequest = objectFactory.createRetrieveDocumentSetRequestType();
//			singleRequest.getDocumentRequest().add(entry);
//			RetrieveDocumentSetResponseType response = iti43PortType.sendRetrieve(singleRequest, toRetrieve.getPatientId());
//			result.merge(xdsTransformer.transform(response, singleRequest, toRetrieve));
//			iti43PortType.addServiceSpecificResults(response, result);
//		}
//		return result;
//	}

}