/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.integrationtest;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.FindDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.responses.Status;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.query.AdhocQueryResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.context.support.AnnotationConfigContextLoader;

import dk.nsi.dds.projects.sxa.common.configuration.IntegrationTestConfiguration;
import dk.nsi.dds.projects.sxa.consumer.DocumentConsumer;
import dk.nsi.dds.projects.sxa.testhelper.SxaQueryHelper;
import dk.nsi.dds.projects.sxa.testhelper.SxaTestDataHelper;

/**
 * This is an Iti18 test
 */
@RunWith(SpringRunner.class)
@ContextConfiguration(classes = { IntegrationTestConfiguration.class}, loader = AnnotationConfigContextLoader.class)
public class SxaErrorsIT {

	@Autowired
	private DocumentConsumer docConsumer;
	@Autowired
	private SxaTestDataHelper testDataHelper;
	@Autowired 
	private SxaQueryHelper queryHelper;

	@Test
	public void noMetadataReturnedForPatientIdContainingAlphanum() throws Exception {
		// Arrange
		FindDocumentsQuery query = queryHelper.prepareQueryForSxaData(testDataHelper.getIllegalPatientId());
		// Act
		AdhocQueryResponse queryResult = docConsumer.doQuery(query);
		// Assert
		Assert.assertNotNull(queryResult.getRegistryObjectList());
		Assert.assertTrue(queryResult.getRegistryObjectList().getIdentifiable().size() == 0);
		Assert.assertTrue(Status.valueOfOpcode(queryResult.getStatus()) == Status.FAILURE);
	}

}
