/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.testhelper;

import org.joda.time.DateTime;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Identifiable;
import org.springframework.stereotype.Component;

import dk.nsi.dds.projects.sxa.common.TestDataHelper;
import dk.nsi.dds.projects.sxa.common.configuration.SXAMappedValue;
import dk.nsi.dds.projects.sxa.common.nationalprofile.NationalProfileValueFactory;

@Component
public class SxaTestDataHelper {

  /**
   * Default patient has appointments as follows:
   * 
   * now ----- plus one day ---- plus five days ---- plus ten days ---- plus twenty days
   *                             (Defined as         (Defined as        (Defined as
   *                              LOWER_WINDOW)       UPPER_WINDOW)      UPPER_BOUNDARY)
   */
  public static final String DEFAULT_PATIENT_CPR = "1212121212";
  private static DateTime DEFAULT_PATIENT_LOWER_WINDOW = DateTime.now().plusDays(5);
  private static DateTime DEFAULT_PATIENT_UPPER_WINDOW = DateTime.now().plusDays(10);
  private static DateTime DEFAULT_PATIENT_UPPER_BOUNDARY = DateTime.now().plusDays(20);
  
  public static final String UNKNOWN_PATIENT_CPR = "1212120000";
  private static final String ILLEGAL_PATIENT_CPR = "251248M111";
  private NationalProfileValueFactory valueFactory = new NationalProfileValueFactory();
  private TestDataHelper practiceSettingCodeHelper = new TestDataHelper();
  
  //public Code practiceSettingCodeForMicroBiology = practiceSettingCodeHelper.practiceSettingCodeForMicrobiology.toCode();
  //public Code practiceSettingCodeForBiochemistry = practiceSettingCodeHelper.practiceSettingCodeForBiochemistry.toCode();
  //public Code practiceSettingCodeForPathology = practiceSettingCodeHelper.practiceSettingCodeForPathology.toCode();
  public SXAMappedValue practiceSettingCodeInvalid = practiceSettingCodeHelper.practiceSettingCodeNonSxa;
  
  public Identifiable getDefaultPatientId() {
    return getPatientId(DEFAULT_PATIENT_CPR);
  }

  public DateTime getDefaultPatientLowerWindow() {
    return DEFAULT_PATIENT_LOWER_WINDOW;
  }

  public DateTime getDefaultPatientUpperWindow() {
    return DEFAULT_PATIENT_UPPER_WINDOW;
  }

  public DateTime getDefaultPatientUpperBoundary() {
    return DEFAULT_PATIENT_UPPER_BOUNDARY;
  }

  public Identifiable getPatientIdForUnknownPatient() {
    return getPatientId(UNKNOWN_PATIENT_CPR);
  }

  public Identifiable getPatientIdForPatientWithNoData() {
    return getPatientId(UNKNOWN_PATIENT_CPR);
  }

  public Identifiable getIllegalPatientId() {
    return getPatientId(ILLEGAL_PATIENT_CPR);
  }
  
  public Identifiable getPatientId(String cpr) {
    return valueFactory.createPatientIdUsingCpr(cpr);
  }
}
