/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.testhelper;

import java.io.BufferedOutputStream;
import java.io.Closeable;
import java.io.File;
import java.io.FileOutputStream;
import java.io.Flushable;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.nio.charset.Charset;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class SxaXmlUtil {
  public static Document parseXmltextToDocument(String xmltext)
    throws ParserConfigurationException, SAXException, IOException {
    return parseXmltextToDocument(xmltext, false);
  }

  public static Document parseXmltextToDocument(String xmltext, boolean forceNamespace)
    throws ParserConfigurationException, SAXException, IOException {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    factory.setNamespaceAware(forceNamespace);
    DocumentBuilder builder = factory.newDocumentBuilder();
    return builder.parse(new InputSource(new StringReader(xmltext)));
  }

  public static String transformDocumentToString(Document doc) throws TransformerException {
    StringWriter writer = new StringWriter();
    Transformer transformer = TransformerFactory.newInstance().newTransformer();
    transformer.transform(new DOMSource(doc), new StreamResult(writer));
    return writer.toString();
  }

  public static Document parseFileToDocument(String fileName, boolean forceNamespaceAware)
    throws ParserConfigurationException, SAXException, IOException {
    InputStream inputStream = null;
    try {
      inputStream = SxaXmlUtil.class.getClassLoader().getResourceAsStream(fileName);
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setNamespaceAware(forceNamespaceAware);
      DocumentBuilder builder = factory.newDocumentBuilder();
      return builder.parse(inputStream);
    } finally {
      close(inputStream);
    }
  }

  public static void writeDocumentToFile(Document document, File outputFile) throws TransformerException, IOException {
    String xmlText = transformDocumentToString(document);
    OutputStreamWriter writer = null;
    try {
      writer = new OutputStreamWriter(new BufferedOutputStream(new FileOutputStream(outputFile)),
        Charset.forName("UTF-8"));
      writer.write(xmlText);
    } finally {
      close(writer);
    }
  }

  private static void close(Closeable closeable) {
    try {
      if (closeable != null) {
        if (closeable instanceof Flushable) {
          flush((Flushable) closeable);
        }
        closeable.close();
      }
    } catch (Exception ex) {
      // Ignore
    }
  }

  private static void flush(Flushable flushable) {
    try {
      if (flushable != null) {
        flushable.flush();
      }
    } catch (Exception ex) {
      // Ignore
    }
  }
}
