package dk.nsi.dds.projects.sxa.labreportservicestub.configuration;

import java.io.IOException;
import java.net.URL;
import java.time.LocalDateTime;
import java.util.Properties;

import javax.annotation.PostConstruct;
import javax.ejb.ConcurrencyManagement;
import javax.ejb.ConcurrencyManagementType;
import javax.ejb.Startup;
import javax.enterprise.inject.Produces;
import javax.inject.Singleton;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

@Singleton
@Startup
@ConcurrencyManagement(ConcurrencyManagementType.BEAN)
public class SxaConfigurationLoader {

	private static final Logger log = Logger.getLogger(SxaConfigurationLoader.class);

	private static SxaConfiguration sxaConfiguration;
	private static String startupTime;

	@PostConstruct
	public void startup() {
		String logDir4TestLocation = System.getProperty("jboss.server.log.dir");
		if(logDir4TestLocation == null) {
			String logDir4Test = System.getProperty("java.io.tmpdir");
			logDir4TestLocation = logDir4Test != null ? logDir4Test : classLoader().getResource(".").getFile();
		}
		
		final String logConfigFile = getSxaConfiguration().getProperty(SxaConstants.LOG_CONFIG_FILE);
		URL resource = classLoader().getResource(logConfigFile);
		//new Log4jConfigurationLoader().loadConfiguration(logConfigFile);
		PropertyConfigurator.configure(resource);
		log.info("Configured Log4j from " + logConfigFile+" @ jboss.server.log.dir="+logDir4TestLocation);
	}

	@Produces
	public SxaConfiguration getSxaConfiguration() {
		System.out.println("SxaConfigurationLoader.getSxaConfiguration called");	
		if (sxaConfiguration == null) {
			System.out.println("sxaConfiguration is null");	
			Properties properties = new Properties();
			try {
				properties.load(classLoader().getResourceAsStream(SxaConstants.SXALABPROVIDERSTUB_CONFIG_FILE));
			} catch (IOException e) {
				throw new RuntimeException("Kunne ikke indlæse properties fra " + SxaConstants.SXALABPROVIDERSTUB_CONFIG_FILE, e);
			}

			System.out.println("Creating new sxaConfiguration");	
			sxaConfiguration = new SxaConfiguration() {
				final Properties props = properties;

				@Override
				public String getProperty(String key) {
					return props.getProperty(key);
				}
			};
		}

		return sxaConfiguration;
	}
	
	private ClassLoader classLoader() {
		return SxaConfigurationLoader.class.getClassLoader();
	}
	
	@Produces String getStartupTime() {
		if(startupTime == null) {
			startupTime = LocalDateTime.now().toString();
		}
		return startupTime;	
	}
}
