/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2012 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in the
 * Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.labreportservicestub.configuration;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.util.HashMap;
import java.util.Map;

public class SxaCprToResponseFileConfiguration {
  private File file;
  private Map<String, String> cprToResponseFilePairs;

  public SxaCprToResponseFileConfiguration(File file) throws IOException {
    cprToResponseFilePairs = new HashMap<String, String>();
    parseFileToMap(file);
    this.file = file;
  }
  
  public SxaCprToResponseFileConfiguration(InputStream input) throws IOException {
    cprToResponseFilePairs = new HashMap<String, String>();
    parseInputStreamToMap(input);
  }

  /**
   * The file parses to key-value pairs with the first two occurrences between a
   * comma
   * 
   * @param file
   * @throws IOException
   */
  protected void parseFileToMap(File file) throws IOException {
    try (BufferedReader reader = Files.newBufferedReader(file.toPath(), Charset.forName("UTF-8"))) {
      Map<String, String> tempKeyValue = new HashMap<String, String>();
      reader.lines().forEach(line -> {
        String[] groups = line.split(",");
        if (groups.length > 1) {
          tempKeyValue.put(groups[0], groups[1]);
        } else if (groups.length == 1) {
          tempKeyValue.put(groups[0], "");
        }
      });
      cprToResponseFilePairs = tempKeyValue;
    }
  }
  
  protected void parseInputStreamToMap(InputStream input) throws IOException {
    try (BufferedReader reader = new BufferedReader(new InputStreamReader(input))) {
      Map<String, String> tempKeyValue = new HashMap<String, String>();
      reader.lines().forEach(line -> {
        String[] groups = line.split(",");
        if (groups.length > 1) {
          tempKeyValue.put(groups[0], groups[1]);
        } else if (groups.length == 1) {
          tempKeyValue.put(groups[0], "");
        }
      });
      cprToResponseFilePairs = tempKeyValue;
    }
  }

  public Map<String, String> getCprToResponseFilePairs() {
    return cprToResponseFilePairs;
  }
  
  public String getFileName() {
    return file.getName();
  }
}
