package dk.nsi.dds.projects.sxa.labreportservicestub.configuration.logic;

import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;
import javax.json.Json;
import javax.json.JsonObjectBuilder;
import javax.json.stream.JsonGenerator;

import org.apache.log4j.Logger;

import dk.nsi.dds.projects.sxa.labreportservicestub.configuration.HttpHeaderControl;
import dk.nsi.dds.projects.sxa.labreportservicestub.configuration.SxaConfiguration;

public class SimpleHttpHeaderControl implements HttpHeaderControl {

	private static final Logger logger = Logger.getLogger(SimpleHttpHeaderControl.class);

	@Inject
	private SxaConfiguration sxaConfiguration;


	@Override
	public boolean doPrint(Map<String, List<String>> httpHeaders) {
		return httpHeaders != null && Boolean.valueOf(sxaConfiguration.getProperty("sxa.labprovider.httpheaders.print"));
	}

	@Override
	public void print(Map<String, List<String>> httpHeaders) {
		final Map<String, Boolean> config = Collections.singletonMap(JsonGenerator.PRETTY_PRINTING, true);
		JsonObjectBuilder headerOutputBuilder = Json.createBuilderFactory(config).createObjectBuilder();

		JsonObjectBuilder headerValuesBuilder = Json.createBuilderFactory(config).createObjectBuilder();
		if (httpHeaders != null) {
			for (String key : httpHeaders.keySet()) {
				List<String> list = httpHeaders.get(key);
				headerValuesBuilder.add(key, String.valueOf(list));
			}
		}

		headerOutputBuilder.add("HTTP_REQUEST_HEADERS", headerValuesBuilder);

		logger.info(headerOutputBuilder.build());

	}

	@Override
	public boolean doValidate(Map<String, List<String>> httpHeaders) {
		return httpHeaders != null && Boolean.valueOf(sxaConfiguration.getProperty("sxa.labprovider.httpheaders.validate"));
	}

	@Override
	public void validate(Map<String, List<String>> httpHeaders) {
		// F.ex. validate headers either content of a header or that presence of header
		//throw new ValidationException("Header is not valid");
	}
}
