/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in the
 * Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.labreportservicestub.configuration.logic;

import java.util.List;

import javax.inject.Inject;
import javax.xml.ws.Holder;

import com.cgi.svareksponering.GetDataFromPeriodParameters;
import com.cgi.svareksponering.ResultatPakkePIH;
import com.cgi.svareksponering.Speciale;

import dk.nsi.dds.projects.sxa.labreportservicestub.exception.SxaException;
import dk.nsi.dds.projects.sxa.labreportservicestub.xml.SxaXmlReader;
import dk.nsi.dds.projects.sxa.labreportservicestub.xml.entity.SxaSoapXmlEnvelope;
import dk.nsi.schema.privacy.pih._1.PrivacyInformationHeaderType;

public class SxaLogic {
	
	@Inject SxaXmlReader sxaXmlReader;
	
  public SxaLogic verifyPatientCpr(String patientCpr) {
    if (patientCpr == null || patientCpr.trim().isEmpty() || patientCpr.compareTo("_") == 0 || patientCpr.length() != 10) {
      throw new SxaException("Patient CPR er ikke korrekt specificeret");
    }
    return this;
  }

  public SxaLogic authenticateUser(String username, String userPassword) {
    if (username == null || username.trim().isEmpty() || username.compareTo("_") == 0) {
      throw new SxaException("Brugeren kunne ikke godkendes");
    }
    if (userPassword == null || userPassword.trim().isEmpty() || userPassword.compareTo("_") == 0) {
      throw new SxaException("Brugeren kunne ikke godkendes");
    }
    return this;
  }

  public ResultatPakkePIH doLogic(GetDataFromPeriodParameters parameters, Holder<PrivacyInformationHeaderType> header)
    throws Exception {
    String patientCpr = parameters.getPatientCpr();
    List<Speciale> specialer = parameters.getSpecialer().getSpeciale();
    SxaSoapXmlEnvelope envelope = getEnvelopeForPatient(patientCpr, specialer);
    setPIH(envelope, header);
    return getResultPakkePIHResponseFromEnvelope(envelope);
  }

  private SxaSoapXmlEnvelope getEnvelopeForPatient(String patientId, List<Speciale> specialer) throws Exception {
    return sxaXmlReader.getEnvelopeFromFile(patientId, specialer);
  }

  private ResultatPakkePIH getResultPakkePIHResponseFromEnvelope(SxaSoapXmlEnvelope envelope) throws Exception {
    return envelope.getBody().getResultatPakkePIH();
  }

  private void setPIH(SxaSoapXmlEnvelope envelope, Holder<PrivacyInformationHeaderType> pih) {
    pih.value = envelope.getHeader().getPrivacyInformationHeader();
  }
}
