/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2014 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsi.dds.projects.sxa.labreportservicestub;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import org.jglue.cdiunit.AdditionalClasses;
import org.jglue.cdiunit.CdiRunner;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.cgi.svareksponering.Speciale;

import dk.nsi.dds.projects.sxa.common.TestDataHelper;
import dk.nsi.dds.projects.sxa.labreportservicestub.configuration.SxaConfigurationLoader;
import dk.nsi.dds.projects.sxa.labreportservicestub.xml.SxaXmlReader;
import dk.nsi.dds.projects.sxa.labreportservicestub.xml.entity.SxaSoapXmlEnvelope;
import org.junit.Assert;

@RunWith(CdiRunner.class)
@AdditionalClasses({SxaConfigurationLoader.class})
public class SxaXmlReaderTest {

  @Inject SxaXmlReader sxaXmlReader;
  
  @Test
  public void readFromXmlFile() throws Exception {
	  
    // Arrange
    String patientCpr = TestDataHelper.DEFAULT_PATIENT_ID;
    List<Speciale> specialityList = new ArrayList<>();
    specialityList.add(Speciale.MIKROBIOLOGI);
    specialityList.add(Speciale.KLINISK_BIOKEMI);
    specialityList.add(Speciale.PATOLOGI);
    
    // Act
    SxaSoapXmlEnvelope result = sxaXmlReader.getEnvelopeFromFile(patientCpr, specialityList);
    
    // Assert
    Assert.assertNotNull(result);
    Assert.assertNotNull(result.getHeader());
    Assert.assertNotNull(result.getHeader().getPrivacyInformationHeader());
    Assert.assertEquals(3, result.getHeader().getPrivacyInformationHeader().getDeclaration().get(0).getSource().getAttribute().size());
    Assert.assertNotNull(result.getHeader().getPrivacyInformationHeader().getDeclaration().get(0).getLocation().getElement().get(0));
    Assert.assertEquals("0805721057", result.getHeader().getPrivacyInformationHeader().getDeclaration().get(0).getSource().getAttribute().get(0).getValue());
    Assert.assertEquals(3, result.getBody().getResultatPakkePIH().getMappedeSvar().getDocumentWrapper().size());
    Assert.assertEquals(3, result.getBody().getResultatPakkePIH().getRaaSvar().getLeverandoerSvar().get(0).getLeverandoerSvarXml().getDocumentWrapper().size());
  }
  
  @Test
  public void readFromEmptyFile() throws Exception {
    // Arrange
    String patientCprForNoSpecialities = TestDataHelper.DEFAULT_PATIENT_ID;
    List<Speciale> specialityList = new ArrayList<>();
    specialityList.add(Speciale.EMPTY);
    
    // Act
    SxaSoapXmlEnvelope result = sxaXmlReader.getEnvelopeFromFile(patientCprForNoSpecialities, specialityList);
    
    // Assert
    Assert.assertNotNull(result);
    Assert.assertEquals(0, result.getBody().getResultatPakkePIH().getMappedeSvar().getDocumentWrapper().size());
  }
}
