package dk.nsi.dds.projects.sxa.labreportservicestub.configuration.logic;

import javax.xml.ws.Holder;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import com.cgi.svareksponering.ArrayOfSpeciale;
import com.cgi.svareksponering.GetDataFromPeriodParameters;
import com.cgi.svareksponering.ResultatPakkePIH;
import com.cgi.svareksponering.Speciale;

import dk.nsi.dds.projects.sxa.labreportservicestub.exception.SxaException;
import dk.nsi.dds.projects.sxa.labreportservicestub.xml.SxaXmlReader;
import dk.nsi.dds.projects.sxa.labreportservicestub.xml.entity.SxaSoapXmlBody;
import dk.nsi.dds.projects.sxa.labreportservicestub.xml.entity.SxaSoapXmlEnvelope;
import dk.nsi.dds.projects.sxa.labreportservicestub.xml.entity.SxaSoapXmlHeader;
import dk.nsi.schema.privacy.pih._1.PrivacyInformationHeaderType;

@RunWith(MockitoJUnitRunner.class)
public class SxaLogicTest {
	@Mock
	GetDataFromPeriodParameters parameters;
	@Mock
	PrivacyInformationHeaderType header;

	@Mock
	SxaSoapXmlEnvelope sxaSoapXmlEnvelope;
	@Mock
	SxaSoapXmlHeader sxaSoapXmlHeader;
	@Mock
	SxaSoapXmlBody sxaSoapXmlBody;
	@Mock
	ResultatPakkePIH resultatPakkePIH;

	@Mock
	SxaXmlReader sxaXmlReader;

	@InjectMocks
	private SxaLogic sxaLogic;

	@Test
	public void testDoLogoc() throws Exception {
		final String patientCpr = "1234567890";
		Mockito.when(parameters.getPatientCpr()).thenReturn(patientCpr);
		ArrayOfSpeciale arrayOfSpeciale = new ArrayOfSpeciale();
		arrayOfSpeciale.getSpeciale().add(Speciale.MIKROBIOLOGI);
		Mockito.when(parameters.getSpecialer()).thenReturn(arrayOfSpeciale);

		Mockito.when(sxaXmlReader.getEnvelopeFromFile(patientCpr, arrayOfSpeciale.getSpeciale())).thenReturn(sxaSoapXmlEnvelope);
		Mockito.when(sxaSoapXmlEnvelope.getHeader()).thenReturn(sxaSoapXmlHeader);
		Mockito.when(sxaSoapXmlEnvelope.getBody()).thenReturn(sxaSoapXmlBody);
		Mockito.when(sxaSoapXmlBody.getResultatPakkePIH()).thenReturn(resultatPakkePIH);

		Assert.assertEquals("Forventede Mocked ResultatPakkePIH tilbage", resultatPakkePIH, sxaLogic.doLogic(parameters, new Holder<PrivacyInformationHeaderType>(header)));
		
		Mockito.verify(parameters, Mockito.times(1)).getPatientCpr();
		Mockito.verify(parameters, Mockito.times(1)).getSpecialer();
		
		Mockito.verify(sxaXmlReader, Mockito.times(1)).getEnvelopeFromFile(patientCpr, arrayOfSpeciale.getSpeciale());
		Mockito.verify(sxaSoapXmlEnvelope, Mockito.times(1)).getHeader();
		Mockito.verify(sxaSoapXmlEnvelope, Mockito.times(1)).getBody();
		Mockito.verify(sxaSoapXmlBody, Mockito.times(1)).getResultatPakkePIH();
		
		Mockito.verifyNoMoreInteractions(parameters, sxaXmlReader, sxaSoapXmlEnvelope, sxaSoapXmlBody, resultatPakkePIH);
	}

	@Test
	public void testAuthenticateUserUsernameAndPasswordOk() {
		sxaLogic.authenticateUser("user", "password");
	}

	@Test(expected = SxaException.class)
	public void testAuthenticateUserUsernameIsNull() {
		sxaLogic.authenticateUser(null, "password");
	}

	@Test(expected = SxaException.class)
	public void testAuthenticateUserUsernameIsUnderscore() {
		sxaLogic.authenticateUser("_", "password");
	}

	@Test(expected = SxaException.class)
	public void testAuthenticateUserUsernameIsSpace() {
		sxaLogic.authenticateUser(" ", "password");
	}

	@Test(expected = SxaException.class)
	public void testAuthenticateUserPasswordIsNull() {
		sxaLogic.authenticateUser("user", null);
	}

	@Test(expected = SxaException.class)
	public void testAuthenticateUserPasswordIsUnderscore() {
		sxaLogic.authenticateUser("user", "_");
	}

	@Test(expected = SxaException.class)
	public void testAuthenticateUserPasswordIsSpace() {
		sxaLogic.authenticateUser("user", " ");
	}

	@Test
	public void testVerifyPatientCprPatientCprOk() {
		sxaLogic.verifyPatientCpr("1234567890");
	}

	@Test(expected = SxaException.class)
	public void testVerifyPatientCprPatientCprIsNull() {
		sxaLogic.verifyPatientCpr(null);
	}

	@Test(expected = SxaException.class)
	public void testVerifyPatientCprPatientCprIsUnderscore() {
		sxaLogic.verifyPatientCpr("_");
	}

	@Test(expected = SxaException.class)
	public void testVerifyPatientCprPatientCprIsSpace() {
		sxaLogic.verifyPatientCpr(" ");
	}

	@Test(expected = SxaException.class)
	public void testVerifyPatientCprPatientCprIs9Characters() {
		sxaLogic.verifyPatientCpr("123456789");
	}

	@Test(expected = SxaException.class)
	public void testVerifyPatientCprPatientCprIs11Characters() {
		sxaLogic.verifyPatientCpr("12345678901");
	}
}
