/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentmetadataprovider.configuration;

import java.io.File;

import org.openehealth.ipf.commons.ihe.xds.core.metadata.Person;

import com.thoughtworks.xstream.XStream;

import dk.nsi.dds.projects.sxa.common.configuration.SXAMappedValue;
import dk.nsi.dds.projects.sxa.common.configuration.XIdentifiable;
import dk.nsi.dds.projects.sxa.common.configuration.XOrganization;
import dk.nsi.dds.projects.sxa.common.util.configloading.FileModifiedWatcher;
import dk.nsi.dds.projects.sxa.common.util.xmlhandling.XStreamFileException;
import dk.nsi.dds.projects.sxa.common.util.xmlhandling.XStreamFileHandler;

public class MetadataConfigurationHandler {
	private XStreamFileHandler xStreamFileHandler;
	private XStreamFileHandler xStreamFilterFileHandler;
	private FileModifiedWatcher configurationFile;
	private FileModifiedWatcher filterFile;
	private MetadataConfiguration metadataConfiguration;
	//private static MetadataFilterConfiguration metadataFilterConfiguration;

	public MetadataConfigurationHandler(File configurationFile, File filterFile) {
		this.configurationFile = new FileModifiedWatcher(configurationFile);
		this.filterFile = new FileModifiedWatcher(filterFile);
		this.xStreamFileHandler = new XStreamFileHandler(configurationFile.getPath(), createXStream());
		this.xStreamFilterFileHandler = new XStreamFileHandler(filterFile.getPath(), createXStream());
	}

	private XStream createXStream() {
		XStream xstream = new XStream();
		xstream.alias("configuration", MetadataConfiguration.class);
		xstream.alias("filter", MetadataFilterConfiguration.class);
		xstream.alias("metadata", MetadataContentConfiguration.class);
		xstream.alias("codedValue", SXAMappedValue.class);
		xstream.alias("person", Person.class);
		xstream.alias("id", XIdentifiable.class);
		xstream.alias("authorInstitution", XOrganization.class);
		return xstream;
	}

	public MetadataConfiguration readConfiguration() throws XStreamFileException {
		if ((this.configurationFile.isModified()) || (this.filterFile.isModified())) {
			synchronized (this) {
				if ((this.configurationFile.isModified()) || (this.filterFile.isModified())) {
					MetadataFilterConfiguration filter = (MetadataFilterConfiguration) this.xStreamFilterFileHandler.readXmlFile(MetadataFilterConfiguration.class);
					MetadataContentConfiguration conf = (MetadataContentConfiguration) this.xStreamFileHandler.readXmlFile(MetadataContentConfiguration.class);
					MetadataConfiguration configuration = new MetadataConfiguration();
					configuration.setMetadataContentConfiguration(conf);
					configuration.setMetadataConfigurationFilter(filter);
					this.metadataConfiguration = configuration;
					this.configurationFile.reload();
					this.filterFile.reload();
				}
			}
		}
		return this.metadataConfiguration;
	}

	public void writeConfiguration(MetadataConfiguration akConfiguration) throws XStreamFileException {
		this.xStreamFileHandler.writeXmlFile(akConfiguration.getMetadataContentConfiguration());
		this.xStreamFilterFileHandler.writeXmlFile(akConfiguration.getMetadataConfigurationFilter());
	}

	public void deleteConfiguration() {
		if (this.configurationFile.getFile().exists()) {
			this.configurationFile.getFile().delete();
		}
		if (this.filterFile.getFile().exists()) {
			this.filterFile.getFile().delete();
		}
	}
}