/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentmetadataprovider.configuration;

import java.util.ArrayList;
import java.util.List;

import org.openehealth.ipf.commons.ihe.xds.core.metadata.Code;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Organization;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Person;

import dk.nsi.dds.projects.sxa.common.configuration.SXAMappedValue;
import dk.nsi.dds.projects.sxa.common.configuration.XOrganization;
import dk.nsi.dds.projects.sxa.common.configuration.XPerson;

public class MetadataContentConfiguration {
	private SXAMappedValue classCode;
	private SXAMappedValue confidentialityCode;
	private List<SXAMappedValue> eventCodes;
	private SXAMappedValue formatCode;
	private SXAMappedValue practiceSettingCode;
	private SXAMappedValue typeCode;
	private SXAMappedValue healthcareFacilityTypeCode;
	
	private XPerson authorPerson;
	private String languageCode;
	private String mimeType;
	private String homeCommunityId;
	private String repositoryUniqueId;
	private String uniqueIdRoot;
	private XOrganization authorInstitution;

	public Person getAuthorPerson() {
		return XPerson.getPerson(authorPerson);
	}

	public void setAuthorPerson(Person authorPerson) {
		this.authorPerson = XPerson.getXPerson(authorPerson);
	}

	public SXAMappedValue getClassCode() {
		return this.classCode;
	}

	public void setClassCode(SXAMappedValue classCode) {
		this.classCode = classCode;
	}

	public SXAMappedValue getConfidentialityCode() {
		return this.confidentialityCode;
	}

	public void setConfidentialityCode(SXAMappedValue confidentiality) {
		this.confidentialityCode = confidentiality;
	}

	public List<SXAMappedValue> getEventCodes() {
		return eventCodes;
	}

	public void setEventCodes(List<SXAMappedValue> eventCodes) {
		this.eventCodes = eventCodes;
	}

	public void addEventCode(SXAMappedValue eventCode) {
		if (eventCode == null) {
			return;
		}
		if (this.eventCodes == null) {
			this.eventCodes = new ArrayList<>();
		}
		
		this.eventCodes.add(eventCode);
	}

	public String getLanguageCode() {
		return this.languageCode;
	}

	public void setLanguageCode(String languageCode) {
		this.languageCode = languageCode;
	}

	public String getMimeType() {
		return this.mimeType;
	}

	public void setMimeType(String mimeType) {
		this.mimeType = mimeType;
	}

	public SXAMappedValue getPracticeSettingCode() {
		return this.practiceSettingCode;
	}

	public void setPracticeSettingCode(Code practiceSettingCode) {
		this.practiceSettingCode = new SXAMappedValue(practiceSettingCode);
	}

	public SXAMappedValue getTypeCode() {
		return this.typeCode;
	}

	public void setTypeCode(SXAMappedValue typeCode) {
		this.typeCode = typeCode;
	}

	public SXAMappedValue getFormatCode() {
		return this.formatCode;
	}

	public void setFormatCode(SXAMappedValue formatCode) {
		this.formatCode = formatCode;
	}

	public String getHomeCommunityId() {
		return this.homeCommunityId;
	}

	public void setHomeCommunityId(String homeCommunityId) {
		this.homeCommunityId = homeCommunityId;
	}

	public String getRepositoryUniqueId() {
		return this.repositoryUniqueId;
	}

	public void setRepositoryUniqueId(String repositoryUniqueId) {
		this.repositoryUniqueId = repositoryUniqueId;
	}

	public String getUniqueIdRoot() {
		return this.uniqueIdRoot;
	}

	public void setUniqueIdRoot(String uniqueIdRoot) {
		this.uniqueIdRoot = uniqueIdRoot;
	}

	public SXAMappedValue getHealthcareFacilityTypeCode() {
		return this.healthcareFacilityTypeCode;
	}

	public void setHealthcareFacilityTypeCode(Code healthcareFacilityTypeCode) {
		this.healthcareFacilityTypeCode = new SXAMappedValue(healthcareFacilityTypeCode);
	}

	public Organization getAuthorInstitution() {
		return XOrganization.getOrganisation(this.authorInstitution);
	}

	public void setAuthorInstitution(Organization authorInstitution) {
		this.authorInstitution = new XOrganization(authorInstitution);
	}
}
