/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentmetadataprovider.configuration;

import java.util.ArrayList;
import java.util.List;

import dk.nsi.dds.projects.sxa.common.configuration.SXAMappedValue;

public class MetadataFilterConfiguration {
	private List<SXAMappedValue> classCodes = new ArrayList<>();
	private List<SXAMappedValue> confidentialityCodes = new ArrayList<>();
	private List<SXAMappedValue> eventCodes = new ArrayList<>();
	private List<SXAMappedValue> formatCodes = new ArrayList<>();
	private List<SXAMappedValue> practiceSettingCodes = new ArrayList<>();
	private List<SXAMappedValue> typeCodes = new ArrayList<>();

	public List<SXAMappedValue> getPracticeSettingCodes() {
		return this.practiceSettingCodes;
	}

	public void setPracticeSettingCodes(List<SXAMappedValue> practiceSettingCodes) {
		this.practiceSettingCodes = createListIfNull(practiceSettingCodes);
	}

	public void addPracticeSettingCodes(SXAMappedValue practiceSettingCode) {
		this.practiceSettingCodes = addCodeToList(this.practiceSettingCodes, practiceSettingCode);
	}

	public List<SXAMappedValue> getFormatCodes() {
		return this.formatCodes;
	}

	public void setFormatCodes(List<SXAMappedValue> formatCodes) {
		this.formatCodes = createListIfNull(formatCodes);
	}

	public void addFormatCodes(SXAMappedValue formatCode) {
		this.formatCodes = addCodeToList(this.formatCodes, formatCode);
	}

	public List<SXAMappedValue> getConfidentialityCodes() {
		return this.confidentialityCodes;
	}

	public void setConfidentialityCodes(List<SXAMappedValue> confidentialityCodes) {
		this.confidentialityCodes = createListIfNull(confidentialityCodes);

	}

	public void addConfidentialityCodes(SXAMappedValue confidentialityCode) {
		this.confidentialityCodes = addCodeToList(this.confidentialityCodes, confidentialityCode);
	}

	public List<SXAMappedValue> getClassCodes() {
		return this.classCodes;
	}

	public void setClassCodes(List<SXAMappedValue> classCodes) {
		this.classCodes = createListIfNull(classCodes);

	}

	public void addClassCode(SXAMappedValue classCode) {
		this.classCodes = addCodeToList(this.classCodes, classCode);
	}

//	public List<Code> getTypeCodes() {
//		return this.typeCodes.stream().map(e -> e.toCode()).collect(Collectors.toList());
//	}
	
	public List<SXAMappedValue> getTypeCodes() {
		return this.typeCodes;
	}

	public void setTypeCodes(List<SXAMappedValue> typeCodes) {
		this.typeCodes = createListIfNull(typeCodes);

	}

	public void addTypeCode(SXAMappedValue typeCode) {
		this.typeCodes = addCodeToList(this.typeCodes, typeCode);
	}

	public List<SXAMappedValue> getEventCodes() {
		return this.eventCodes;
	}

	public void setEventCodes(List<SXAMappedValue> eventCodes) {
		this.eventCodes = createListIfNull(eventCodes);

	}

	public void addEventCodes(SXAMappedValue eventCode) {
		this.eventCodes = addCodeToList(this.eventCodes, eventCode);
	}

	private List<SXAMappedValue> createListIfNull(List<SXAMappedValue> list){
		return list != null ? list : new ArrayList<>();
	}
	
	private List<SXAMappedValue> addCodeToList(List<SXAMappedValue> list, SXAMappedValue value) {
		List<SXAMappedValue> result = createListIfNull(list);
		
		if (value != null && !result.contains(value)) {
			result.add(value);
		}
		return result;
	}
}
