/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentmetadataprovider.impl;

import org.apache.log4j.Logger;
import org.openehealth.ipf.commons.ihe.xds.core.requests.QueryRegistry;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.FindDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.responses.ErrorCode;
import org.openehealth.ipf.commons.ihe.xds.core.responses.ErrorInfo;
import org.openehealth.ipf.commons.ihe.xds.core.responses.QueryResponse;
import org.openehealth.ipf.commons.ihe.xds.core.responses.Severity;

import dk.nsi.dds.projects.sxa.common.util.perflogger.PerformanceLogger;
import dk.nsi.dds.projects.sxa.common.util.perflogger.PerformanceLogger.Monitor;
import dk.nsi.dds.projects.sxa.documentmetadataprovider.ws.DocumentMetadataProvider;

public class MetadataProviderQueryImpl implements MetadataProviderQuery {
	private static final Logger logger = Logger.getLogger(MetadataProviderQueryImpl.class);

	private DocumentMetadataProvider documentMetadataProvider;

	public MetadataProviderQueryImpl(DocumentMetadataProvider documentMetadataProvider) {
		this.documentMetadataProvider = documentMetadataProvider;
	}

	@Override
	public QueryResponse handleRequest(QueryRegistry registry) {
		logger.debug("handleRequest: " + registry);
		try (Monitor monitor = new PerformanceLogger.Monitor("MetadataProviderQueryImpl.handleRequest")) {
			if (registry != null && registry.getQuery() instanceof FindDocumentsQuery) {
				FindDocumentsQuery findDocumentsQuery = (FindDocumentsQuery) registry.getQuery();
				return documentMetadataProvider.doQuery(findDocumentsQuery);
			} else {
				return createAndAddStandarError(ErrorCode.UNKNOWN_STORED_QUERY, documentMetadataProvider.getCodeContext(), Severity.WARNING);
			}
		}
	}
	
	public static QueryResponse createAndAddCustomError(String customErrorCode, String errorCodeContext, Severity severity) {
		return createAndAddCustomError(new QueryResponse(), customErrorCode, errorCodeContext, severity);
	}
	
	public static QueryResponse createAndAddCustomError(QueryResponse response, String customErrorCode, String errorCodeContext, Severity severity) {
		ErrorInfo error = new ErrorInfo();
		error.setErrorCode(ErrorCode._USER_DEFINED);
		error.setCustomErrorCode(customErrorCode);
		error.setSeverity(severity);
		error.setCodeContext(errorCodeContext);
		
		response.getErrors().add(error);
		return response;
	}
	
	public static QueryResponse createAndAddStandarError(ErrorCode errorCode, String errorCodeContext, Severity severity) {
		return createAndAddStandarError(new QueryResponse(), errorCode, errorCodeContext, severity);
	}
	
	public static QueryResponse createAndAddStandarError(QueryResponse response, ErrorCode errorCode, String errorCodeContext, Severity severity) {
		ErrorInfo error = new ErrorInfo();
		error.setErrorCode(errorCode);
		error.setSeverity(severity);
		error.setCodeContext(errorCodeContext);
		
		response.getErrors().add(error);
		return response;
	}
}
