/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentmetadataprovider;

import org.openehealth.ipf.commons.ihe.xds.core.requests.QueryRegistry;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.FindDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.responses.QueryResponse;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.query.AdhocQueryRequest;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.query.AdhocQueryResponse;

import dk.nsi.dds.projects.sxa.common.xds.UnexpectedDataException;
import dk.nsi.dds.projects.sxa.common.xds.UnexpectedEncodingException;
import dk.nsi.dds.projects.sxa.common.xds.impl.AdhocQueryRequestBuilder;
import dk.nsi.dds.projects.sxa.documentmetadataprovider.impl.MetadataProviderQuery;
import dk.nsi.dds.projects.sxa.documentmetadataprovider.impl.MetadataProviderQueryImpl;
import dk.nsi.dds.projects.sxa.documentmetadataprovider.ws.DocumentMetadataProvider;
import dk.nsi.dds.projects.sxa.documentmetadataprovider.ws.MetadataProviderWS;

public class DocumentMetadataProviderHelper {

	public AdhocQueryResponse doRequest(FindDocumentsQuery query, DocumentMetadataProvider documentMetadataProvider) throws UnexpectedDataException, UnexpectedEncodingException {

		MetadataProviderQuery metadataProviderQuery = new MetadataProviderQueryImpl(documentMetadataProvider);

		AdhocQueryRequest adhocQueryRequest = new AdhocQueryRequestBuilder().createRegistryStoredQueryFindDocumentsRequest(query);

		QueryRegistry queryRegistry = MetadataProviderWS.toRegistry(adhocQueryRequest);
		QueryResponse queryResponse = metadataProviderQuery.handleRequest(queryRegistry);
		return MetadataProviderWS.convert(queryResponse);
	}
}