/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentmetadataprovider.configuration;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import dk.nsi.dds.projects.sxa.common.configuration.SXAMappedValue;


public class MetadataFilterConfigurationTest {
	public final SXAMappedValue mappedValue = new SXAMappedValue("code", "codesystem");
	
	public MetadataFilterConfiguration metadataFilterConfiguration;
	
	@Before
	public void setup() {
		metadataFilterConfiguration = new MetadataFilterConfiguration();
	}
	
	@Test
	public void testClassCode() {
		metadataFilterConfiguration.addClassCode(null);
		
		Assert.assertEquals(0, metadataFilterConfiguration.getClassCodes().size());
		
		metadataFilterConfiguration.addClassCode(mappedValue);
		Assert.assertEquals(1, metadataFilterConfiguration.getClassCodes().size());
		
		metadataFilterConfiguration.setClassCodes(null);
		Assert.assertEquals(0, metadataFilterConfiguration.getClassCodes().size());
	}
	
	@Test
	public void testConfidentialityCodes() {
		metadataFilterConfiguration.addConfidentialityCodes(null);
		
		Assert.assertEquals(0, metadataFilterConfiguration.getConfidentialityCodes().size());
		
		metadataFilterConfiguration.addConfidentialityCodes(mappedValue);
		Assert.assertEquals(1, metadataFilterConfiguration.getConfidentialityCodes().size());
		
		metadataFilterConfiguration.setConfidentialityCodes(null);
		Assert.assertEquals(0, metadataFilterConfiguration.getConfidentialityCodes().size());
	}
	
	@Test
	public void testEventCodes() {
		metadataFilterConfiguration.addEventCodes(null);
		
		Assert.assertEquals(0, metadataFilterConfiguration.getEventCodes().size());
		
		metadataFilterConfiguration.addEventCodes(mappedValue);
		Assert.assertEquals(1, metadataFilterConfiguration.getEventCodes().size());
		
		metadataFilterConfiguration.setEventCodes(null);
		Assert.assertEquals(0, metadataFilterConfiguration.getEventCodes().size());
	}
	
	@Test
	public void testFormatCodes() {
		metadataFilterConfiguration.addFormatCodes(null);
		
		Assert.assertEquals(0, metadataFilterConfiguration.getFormatCodes().size());
		
		metadataFilterConfiguration.addFormatCodes(mappedValue);
		Assert.assertEquals(1, metadataFilterConfiguration.getFormatCodes().size());
		
		metadataFilterConfiguration.setFormatCodes(null);
		Assert.assertEquals(0, metadataFilterConfiguration.getFormatCodes().size());
	}
	
	@Test
	public void testPracticeSettingCodes() {
		metadataFilterConfiguration.addPracticeSettingCodes(null);
		
		Assert.assertEquals(0, metadataFilterConfiguration.getPracticeSettingCodes().size());
		
		metadataFilterConfiguration.addPracticeSettingCodes(mappedValue);
		Assert.assertEquals(1, metadataFilterConfiguration.getPracticeSettingCodes().size());
		
		metadataFilterConfiguration.setPracticeSettingCodes(null);
		Assert.assertEquals(0, metadataFilterConfiguration.getPracticeSettingCodes().size());
	}
	
	@Test
	public void testTypeCode() {
		metadataFilterConfiguration.addTypeCode(null);
		
		Assert.assertEquals(0, metadataFilterConfiguration.getTypeCodes().size());
		
		metadataFilterConfiguration.addTypeCode(mappedValue);
		Assert.assertEquals(1, metadataFilterConfiguration.getTypeCodes().size());
		
		metadataFilterConfiguration.setTypeCodes(null);
		Assert.assertEquals(0, metadataFilterConfiguration.getTypeCodes().size());
	}

}
