/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentmetadataprovider.impl;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;
import org.openehealth.ipf.commons.ihe.xds.core.requests.QueryRegistry;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.FindDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.GetAllQuery;
import org.openehealth.ipf.commons.ihe.xds.core.responses.ErrorCode;
import org.openehealth.ipf.commons.ihe.xds.core.responses.QueryResponse;

import dk.nsi.dds.projects.sxa.documentmetadataprovider.configuration.MetadataConfiguration;
import dk.nsi.dds.projects.sxa.documentmetadataprovider.ws.DocumentMetadataProvider;

@RunWith(MockitoJUnitRunner.class)
public class MetadataProviderQueryTest {
	
	@Mock
	public MetadataConfiguration metadataConfiguration;

	@InjectMocks
	public DocumentMetadataProvider documentMetadataProvider = new DocumentMetadataProviderImpl();
	
	public MetadataProviderQuery metadataProviderQuery;

	@Before
	public void setup() {
		metadataProviderQuery = new MetadataProviderQueryImpl(documentMetadataProvider);
	}

	@Test
	public void testQueryRegistryIsNull() {
		QueryResponse handleRequest = metadataProviderQuery.handleRequest(null);

		Assert.assertNotNull(handleRequest);
		Assert.assertEquals(0, handleRequest.getDocumentEntries().size());
		Assert.assertEquals(1, handleRequest.getErrors().size());

		Assert.assertEquals(ErrorCode.UNKNOWN_STORED_QUERY, handleRequest.getErrors().get(0).getErrorCode());
	}

	@Test
	public void testQueryRegistryWithoutQuery() {
		QueryRegistry queryRegistry = new QueryRegistry();
		QueryResponse handleRequest = metadataProviderQuery.handleRequest(queryRegistry);

		Assert.assertNotNull(handleRequest);
		Assert.assertEquals(0, handleRequest.getDocumentEntries().size());
		Assert.assertEquals(1, handleRequest.getErrors().size());

		Assert.assertEquals(ErrorCode.UNKNOWN_STORED_QUERY, handleRequest.getErrors().get(0).getErrorCode());
	}

	@Test
	public void testQueryRegistryWithWrongQueryType() {
		GetAllQuery query = new GetAllQuery();
		QueryRegistry queryRegistry = new QueryRegistry(query);
		QueryResponse handleRequest = metadataProviderQuery.handleRequest(queryRegistry);

		Assert.assertNotNull(handleRequest);
		Assert.assertEquals(0, handleRequest.getDocumentEntries().size());
		Assert.assertEquals(1, handleRequest.getErrors().size());

		Assert.assertEquals(ErrorCode.UNKNOWN_STORED_QUERY, handleRequest.getErrors().get(0).getErrorCode());
	}
	
	@Test
	public void testQueryRegistry() {
		Mockito.when(metadataConfiguration.getMetadataConfigurationFilter()).thenThrow(new RuntimeException("Fejl"));
		
		FindDocumentsQuery query = new FindDocumentsQuery();
		QueryRegistry queryRegistry = new QueryRegistry(query);
		QueryResponse handleRequest = metadataProviderQuery.handleRequest(queryRegistry);

		Assert.assertNotNull(handleRequest);
		Assert.assertEquals(0, handleRequest.getDocumentEntries().size());
		Assert.assertEquals(1, handleRequest.getErrors().size());

		Assert.assertEquals(ErrorCode._USER_DEFINED, handleRequest.getErrors().get(0).getErrorCode());
		Assert.assertEquals("Fejl", handleRequest.getErrors().get(0).getCustomErrorCode());
	}

}
