/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentprovider;

import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocument;
import org.openehealth.ipf.commons.ihe.xds.core.responses.ErrorCode;
import org.openehealth.ipf.commons.ihe.xds.core.responses.ErrorInfo;
import org.openehealth.ipf.commons.ihe.xds.core.responses.Severity;

public class ErrorInfoBuilder {

	public static final String XDS_REGISTRYERROR_CODECONTEXT_HOME_COMMUNITY_ID = "Home Community Id: ";
	public static final String XDS_REGISTRYERROR_CODECONTEXT_REPOSITORY_UNIQUE_ID = "Repository Unique Id: ";
	public static final String XDS_REGISTRYERROR_CODECONTEXT_DOCUMENT_UNIQUE_ID = "Document Unique Id: ";

	private final ErrorInfo errorInfo;

	public ErrorInfoBuilder() {
		errorInfo = new ErrorInfo();
	}

	public ErrorInfoBuilder codeContext(String codeContext) {
		errorInfo.setCodeContext(codeContext);
		return this;
	}

	public ErrorInfoBuilder codeContext(RetrieveDocument doc) {
		String codeContext = doc != null ? createCodeContextFormat(doc.getDocumentUniqueId(), doc.getHomeCommunityId(), doc.getRepositoryUniqueId()) : null;
		return codeContext(codeContext);
	}

	public ErrorInfoBuilder codeContext(String documentUniqueId, String homeCommunityId, String repositoryUniqueId) {
		String codeContext = createCodeContextFormat(documentUniqueId, homeCommunityId, repositoryUniqueId);
		return codeContext(codeContext);
	}

	public static String createCodeContextFormat(String documentUniqueId, String homeCommunityId, String repositoryUniqueId) {
		return String.format("[%s, %s, %s]", XDS_REGISTRYERROR_CODECONTEXT_DOCUMENT_UNIQUE_ID + documentUniqueId, XDS_REGISTRYERROR_CODECONTEXT_HOME_COMMUNITY_ID + homeCommunityId,
				XDS_REGISTRYERROR_CODECONTEXT_REPOSITORY_UNIQUE_ID + repositoryUniqueId);
	}

	public ErrorInfoBuilder customErrorCode(String customErrorCode) {
		errorInfo.setCustomErrorCode(customErrorCode);
		return this;
	}

	public ErrorInfoBuilder errorCode(ErrorCode errorCode) {
		errorInfo.setErrorCode(errorCode);
		return this;
	}

	public ErrorInfoBuilder location(String location) {
		errorInfo.setLocation(location);
		return this;
	}

	public ErrorInfoBuilder severity(Severity severity) {
		errorInfo.setSeverity(severity);
		return this;
	}

	public ErrorInfo build() {
		return errorInfo;
	}
}

