/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentprovider.impl;

import java.io.File;
import java.net.URISyntaxException;
import java.net.URL;

import com.thoughtworks.xstream.XStream;

import dk.nsi.dds.projects.sxa.common.util.configloading.FileModifiedWatcher;
import dk.nsi.dds.projects.sxa.common.util.xmlhandling.XStreamFileException;
import dk.nsi.dds.projects.sxa.common.util.xmlhandling.XStreamFileHandler;

public class ProviderConfigurationHandler {
	private FileModifiedWatcher configurationFile;
	private XStreamFileHandler fileHandler;
	private ProviderConfiguration providerConfigurationInstance;

	public ProviderConfigurationHandler(String filename) throws URISyntaxException {
		URL url = this.getClass().getClassLoader().getResource(filename);
		this.configurationFile = new FileModifiedWatcher(new File(url.toURI()));
		fileHandler = new XStreamFileHandler(configurationFile.getFile(), createXStreamObject());
	}

	public ProviderConfigurationHandler(File configurationFile) {
		this.configurationFile = new FileModifiedWatcher(configurationFile);
		fileHandler = new XStreamFileHandler(configurationFile, createXStreamObject());
	}

	private XStream createXStreamObject() {
		XStream xstream = new XStream();
		xstream.alias(ProviderConfiguration.class.getSimpleName(), ProviderConfiguration.class);
		return xstream;
	}

	public ProviderConfiguration readConfiguration() throws XStreamFileException {
		if (configurationFile.isModified()) {
			synchronized (configurationFile) {
				if (configurationFile.isModified()) {
					providerConfigurationInstance = fileHandler.readXmlFile(ProviderConfiguration.class);
					configurationFile.reload();
				}
			}
		}
		return providerConfigurationInstance;
	}

	public void writeConfiguration(ProviderConfiguration providerConfiguration) throws XStreamFileException {
		fileHandler.writeXmlFile(providerConfiguration);
	}
}
