package dk.nsi.dds.projects.sxa.documentprovider.interceptor;

import java.util.Arrays;
import java.util.List;

import javax.inject.Inject;
import javax.interceptor.AroundInvoke;
import javax.interceptor.InvocationContext;
import javax.servlet.http.HttpServletRequest;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;

import dk.nsi.dds.projects.sxa.common.configuration.SXAConfigKeys;
import dk.nsi.dds.projects.sxa.documentprovider.context.HttpHeaderContext;
import dk.nsi.util.Config;

public class HttpHeaderReaderInterceptor {
    @Inject
    private WebServiceContext webServiceContext;

    @Inject
    private HttpHeaderContext httpHeaderContext;

    @Inject
    private Config config;

    private List<String> headersToTransfer;

    @AroundInvoke
    public Object inspectHeaders(InvocationContext context) throws Exception {
        // Initialize
        initializeIfRequired();

        // Extract the headers
        transferHttpHeadersFromRequest();

        // Proceed with the invocation
        return context.proceed();
    }

    private void initializeIfRequired() {
        if(headersToTransfer == null) {
            initialize();
        }
    }

    private void initialize() {
        // Get headers to transfer from config
        String headerProperty = config.getProperty(SXAConfigKeys.SXA_HEADERS);

        if(headerProperty == null) {
            throw new IllegalStateException(String.format("Property %s was not provided!", SXAConfigKeys.SXA_HEADERS));
        }

        headersToTransfer = Arrays.asList(headerProperty.split(","));
    }

    private void transferHttpHeadersFromRequest() {
        HttpServletRequest request = getRequest();

        for(String header : headersToTransfer) {
            String headerValue = request.getHeader(header);
            if(headerValue == null || header.isEmpty()) {
                continue;
            }

            httpHeaderContext.addHttpHeader(header, headerValue);
        }
    }

    private HttpServletRequest getRequest() {
        MessageContext messageContext = extractMessageContext();
        return extractRequest(messageContext);
    }

    private MessageContext extractMessageContext() {
        if(webServiceContext == null) {
            throw new IllegalStateException("webServiceContext was null!");
        }

        MessageContext messageContext = webServiceContext.getMessageContext();
        if(messageContext == null) {
            throw new IllegalStateException("No MessageContext was present on webServiceContext!");
        }

        return messageContext;
    }

    private HttpServletRequest extractRequest(MessageContext messageContext) {
        if(!messageContext.containsKey("HTTP.REQUEST") || !(messageContext.get("HTTP.REQUEST") instanceof HttpServletRequest)) {
            throw new IllegalStateException("Could not extract http request!");
        }

        HttpServletRequest request = (HttpServletRequest)messageContext.get("HTTP.REQUEST");
        return request;
    }
}
