/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentprovider.invoker;

import java.util.Arrays;
import java.util.List;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import org.joda.time.DateTime;

import com.cgi.svareksponering.ArrayOfSpeciale;
import com.cgi.svareksponering.GetDataFromPeriodParameters;
import com.cgi.svareksponering.Speciale;

import dk.nsi.dds.projects.sxa.documentprovider.util.SXAInvocationAttributes;

public class GetDataFromPeriodParametersFactory {
  private DatatypeFactory dataTypeFactory;
  
  public GetDataFromPeriodParametersFactory(DatatypeFactory datatypeFactory) {
    this.dataTypeFactory = datatypeFactory;
  }
  
  public GetDataFromPeriodParameters buildRequest(SXAInvocationAttributes attributes) {
    GetDataFromPeriodParameters getDataFromPeriodParameters = new GetDataFromPeriodParameters();
    getDataFromPeriodParameters.setBrugerType(attributes.getBrugerType());
    getDataFromPeriodParameters.setFraDato(toXmlGregorianCalendar(attributes.getFraDato()));
    getDataFromPeriodParameters.setTilDato(toXmlGregorianCalendar(attributes.getTilDato()));
    getDataFromPeriodParameters.setLoginPassword(attributes.getLoginPassword());
    getDataFromPeriodParameters.setLoginUsername(attributes.getLoginUsername());
    getDataFromPeriodParameters.setPatientCpr((attributes.getPatientCpr()));
    getDataFromPeriodParameters.setRelation(attributes.getRelation());
    // Samtykke
    getDataFromPeriodParameters.setSamtykkeKode(attributes.getSamtykkeKode());
    getDataFromPeriodParameters.setSamtykkeTekst(attributes.getSamtykkeTekst());
    // Slutbruger
    getDataFromPeriodParameters.setSlutbrugerAutorisationsKode(attributes.getSlutbrugerAutorisationsKode());
    getDataFromPeriodParameters.setSlutbrugerCpr(attributes.getSlutbrugerCpr());
    getDataFromPeriodParameters.setSlutbrugerEfternavn(attributes.getSlutbrugerEfternavn());
    getDataFromPeriodParameters.setSlutbrugerEmail(attributes.getSlutbrugerEmail());
    getDataFromPeriodParameters.setSlutbrugerFornavn(attributes.getSlutbrugerFornavn());
    getDataFromPeriodParameters.setSlutbrugerRolle(attributes.getSlutbrugerRolle());
    getDataFromPeriodParameters.setSlutbrugerStilling(attributes.getSlutbrugerStilling());
    getDataFromPeriodParameters.setSlutBrugerIdentifikation(attributes.getSlutBrugerIdentifikation());
    getDataFromPeriodParameters.setSlutBrugerOrganisation(attributes.getSlutBrugerOrganisation());
    getDataFromPeriodParameters.setSpecialer(createSpecialeList(attributes.getSpecialer()));
    return getDataFromPeriodParameters;
  }

  private static ArrayOfSpeciale createSpecialeList(Speciale[] specialer) {
    ArrayOfSpeciale arrayOfSpeciale = new ArrayOfSpeciale();
    List<Speciale> specialerInner = arrayOfSpeciale.getSpeciale();
    Arrays.stream(specialer).forEach(speciale -> specialerInner.add(speciale));
    return arrayOfSpeciale;
  }

  private XMLGregorianCalendar toXmlGregorianCalendar(DateTime time) {
    return dataTypeFactory.newXMLGregorianCalendar(time.toGregorianCalendar());
  }
}
