/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2012 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in the
 * Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentprovider.invoker;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;

import dk.nsi.dds.projects.sxa.common.configuration.SXAConfigKeys;
import org.apache.log4j.Logger;

import com.cgi.svareksponering.ISvarEksponering;
import com.cgi.svareksponering.SvarEksponering;

import dk.nsi.util.Config;

public class SXAServiceInvokerFactory {
	private static final Logger logger = Logger.getLogger(SXAServiceInvokerFactory.class);
	
	
	private String endpoint_url;
	private URL wsdl_url;
	private int requestTimeoutInSeconds;
	private ISvarEksponering port;

	public void initialize(Config config) {
		loadConfig(config);
		createService();
	}

	private void loadConfig(Config config) {
		this.requestTimeoutInSeconds = Integer.parseInt(config.getProperty(SXAConfigKeys.REQUEST_TIMEOUT_SECONDS));
		this.endpoint_url = config.getProperty(SXAConfigKeys.ENDPOINT_URL);
		logger.debug("Loading config in SXAServiceInvokerFactory. Setting endpoint_url to "+endpoint_url);
		
		String wsdl_url_property = config.getProperty(SXAConfigKeys.WSDL_URL);
		try {
			this.wsdl_url = new URL(wsdl_url_property);
			logger.info("Loading config in SXAServiceInvokerFactory. Setting wsdl_url to "+wsdl_url+" from property "+ SXAConfigKeys.WSDL_URL);
		} catch (MalformedURLException e) {
			logger.error("Malformed wsdl url: " + wsdl_url_property, e);
			throw new IllegalArgumentException("Inappropriate WSDL for lab service.");
		}
	}

	private void createService() {
		try {
      		port = new SvarEksponering(wsdl_url, new QName("http://cgi.com/SvarEksponering", "SvarEksponering")).getBasicHttpBindingISvarEksponering();
			setupRequestContext();
		} catch (Exception e) {
			logger.warn("Cannot create client for SvarEksponering service" + e.getMessage());
			if (logger.isDebugEnabled()) {
				logger.debug("Cannot create client for SvarEksponering service", e);
			}
			port = null;
		}
	}
	
	private void setupRequestContext() {
		Map<String, Object> requestContext = ((BindingProvider) port).getRequestContext();
		requestContext.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, endpoint_url);
		requestContext.put("javax.xml.ws.client.connectionTimeout", requestTimeoutInSeconds * 1000);
		requestContext.put("javax.xml.ws.client.receiveTimeout", requestTimeoutInSeconds * 1000);
	}

	public ISvarEksponering getPort() {
		if (port == null) {
			createService();
		}
		return port;
	}
}
