/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentprovider.invoker;

import java.net.SocketTimeoutException;

import javax.servlet.http.HttpServletResponse;
import javax.xml.ws.WebServiceException;
import javax.xml.ws.http.HTTPException;

public class ServiceResponseChecker {

  public static boolean isTimeoutOrServiceUnavailable(WebServiceException we) {
    return we.getCause() != null &&
        (we.getCause() instanceof SocketTimeoutException || isServiceUnavailable(we));
  }

  public static boolean isServiceUnavailable(WebServiceException we) {
    return we.getCause() != null && 
        ((we.getCause() instanceof org.apache.cxf.transport.http.HTTPException &&
            ((org.apache.cxf.transport.http.HTTPException)we.getCause()).getResponseCode() == HttpServletResponse.SC_SERVICE_UNAVAILABLE) ||
            (we.getCause() instanceof HTTPException && ((HTTPException)we.getCause()).getStatusCode() == HttpServletResponse.SC_SERVICE_UNAVAILABLE));
  }

  public static boolean isServiceNotFound(WebServiceException we) {
    return we.getCause() != null &&
        ((we.getCause() instanceof org.apache.cxf.transport.http.HTTPException &&
            (( org.apache.cxf.transport.http.HTTPException)we.getCause()).getResponseCode() == HttpServletResponse.SC_NOT_FOUND) ||
            (we.getCause() instanceof HTTPException && ((HTTPException)we.getCause()).getStatusCode() == HttpServletResponse.SC_NOT_FOUND));
  }

}
