/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentprovider.ws;

import static javax.xml.ws.soap.SOAPBinding.SOAP12HTTP_MTOM_BINDING;

import javax.ejb.EJB;
import javax.interceptor.Interceptors;
import javax.jws.WebService;
import javax.xml.ws.BindingType;
import javax.xml.ws.soap.Addressing;

import dk.nsi.dds.projects.sxa.common.security.AccessValidationInterceptor;
import dk.nsi.dds.projects.sxa.documentprovider.audit.BuildLogRecord;
import dk.nsi.dds.projects.sxa.documentprovider.audit.LogRecordHeaderInterceptor;
import org.apache.log4j.Logger;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.EbXMLRetrieveDocumentSetResponse;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLFactory30;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLRetrieveDocumentSetRequest30;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetRequestType;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetResponseType;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.responses.RetrievedDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.transform.requests.RetrieveDocumentSetRequestTransformer;
import org.openehealth.ipf.commons.ihe.xds.core.transform.responses.RetrieveDocumentSetResponseTransformer;
import org.openehealth.ipf.commons.ihe.xds.iti43.Iti43PortType;

import dk.nsi.dds.projects.sxa.common.util.perflogger.PerformanceLogger;
import dk.nsi.dds.projects.sxa.common.util.perflogger.PerformanceLogger.Monitor;
import dk.nsi.dds.projects.sxa.documentprovider.DocumentRetriever;
import dk.nsi.dds.projects.sxa.documentprovider.impl.ProviderRequestHandler;
import dk.nsi.dds.projects.sxa.documentprovider.impl.ProviderRequestHandlerImpl;

@WebService(serviceName = "iti43")
@BindingType(value = SOAP12HTTP_MTOM_BINDING)
@Addressing(enabled = true, required = false)
public class DocumentProviderWS implements Iti43PortType {

	private static Logger logger = Logger.getLogger(DocumentProviderWS.class);;

	@EJB
	private DocumentRetriever documentRetriever;

	@Override
	@Interceptors({AccessValidationInterceptor.class})
	@BuildLogRecord
	public RetrieveDocumentSetResponseType documentRepositoryRetrieveDocumentSet(RetrieveDocumentSetRequestType body) {
		try (Monitor monitor = new PerformanceLogger.Monitor("DocumentProviderWS.documentRepositoryRetrieveDocumentSet")) {
			logger.debug("Calling iti43 endpoint request:" + body);
			logger.debug("getDocumentRequest:" + body.getDocumentRequest().size());
			RetrieveDocumentSet retrieveDocumentSet = toRetrieveDocumentSet(body);
			ProviderRequestHandler providerRequestHandler = new ProviderRequestHandlerImpl(documentRetriever);
			RetrievedDocumentSet retrievedDocumentSet = providerRequestHandler.handleRequest(retrieveDocumentSet);
			RetrieveDocumentSetResponseType retrieveDocumentSetResponseType = convert(retrievedDocumentSet);
			logger.debug("Done iti43 response:" + retrieveDocumentSetResponseType);
			return retrieveDocumentSetResponseType;
		}
	}

	private static RetrieveDocumentSet toRetrieveDocumentSet(RetrieveDocumentSetRequestType request) {
		EbXMLFactory30 ebXMLFactory30 = new EbXMLFactory30();
		EbXMLRetrieveDocumentSetRequest30 ebXMLRetrieveDocumentSetRequest30 = new EbXMLRetrieveDocumentSetRequest30(request);
		RetrieveDocumentSet retrieveDocumentSet = new RetrieveDocumentSetRequestTransformer(ebXMLFactory30).fromEbXML(ebXMLRetrieveDocumentSetRequest30);
		return retrieveDocumentSet;
	}

	private static RetrieveDocumentSetResponseType convert(RetrievedDocumentSet response) {
		EbXMLFactory30 ebXMLFactory30 = new EbXMLFactory30();
		EbXMLRetrieveDocumentSetResponse ebXMLRetrieveDocumentSetResponse = new RetrieveDocumentSetResponseTransformer(ebXMLFactory30).toEbXML(response);
		return (RetrieveDocumentSetResponseType) ebXMLRetrieveDocumentSetResponse.getInternal();
	}
}
