package dk.nsi.dds.projects.sxa.documentprovider.audit;

import dk.medcom.dgws._2006._04.dgws_1_0.Linking;
import dk.nsi.dds.projects.sxa.common.configuration.SXAConfigKeys;
import dk.nsi.dds.projects.sxa.documentprovider.context.HttpHeaderContext;
import dk.nsi.dds.projects.sxa.documentprovider.util.HsuidHeaderUtil;
import dk.nsi.hsuid._2016._08.hsuid_1_1.HsuidHeader;
import dk.nsi.util.Config;
import loginfo.healthcare._2017_03_01.LogRecord;
import loginfo.healthcare._2017_03_01.TraceInfoType;
import org.apache.cxf.binding.soap.SoapHeader;
import org.apache.cxf.headers.Header;
import org.apache.cxf.jaxb.JAXBDataBinding;
import org.jglue.cdiunit.CdiRunner;
import org.jglue.cdiunit.internal.servlet.MockHttpServletRequestImpl;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetRequestType;
import org.w3c.dom.Document;

import javax.enterprise.inject.Produces;
import javax.inject.Inject;
import javax.interceptor.InvocationContext;
import javax.xml.bind.JAXBContext;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;

import java.util.ArrayList;
import java.util.List;


@RunWith(CdiRunner.class)
public class LogRecordHeaderInterceptorTest {
    private LogRecordHeaderInterceptor subject;

    @Produces
    @Mock
    private HttpHeaderContext httpHeaderContext;

    @Produces
    @Mock
    private WebServiceContext webServiceContext;

    @Produces
    @Mock
    private LogRecordBuilder logRecordBuilder;

    @Produces
    @Mock
    private Config config;

    @Mock
    private InvocationContext invocationContext;

    @Before
    public void setup() {
        subject = new LogRecordHeaderInterceptor(webServiceContext, httpHeaderContext, config, logRecordBuilder);

        LogRecord logRecord = new LogRecord();
        logRecord.setTraceInfo(new TraceInfoType());
        logRecord.getTraceInfo().setRequestId("22222222-2222-2222-2222-222222222222");
        Mockito.when(logRecordBuilder.buildLogRecord(Mockito.any(), Mockito.any(), Mockito.any(), Mockito.any())).thenReturn(logRecord);

        Mockito.when(config.getProperty(SXAConfigKeys.SXA_LOGRECORD_INBOUND_HEADER_NAME)).thenReturn("healthcare.loginfo.logdata");
        Mockito.when(config.getProperty(SXAConfigKeys.SXA_LOGRECORD_OUTBOUND_HEADER_NAME)).thenReturn("healthcare.loginfo.logdata");
    }

    @Test
    public void generateLogRecordHeader_headerIsOnHeaderContext() throws Exception {
        // Arrange
        Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        JAXBContext.newInstance(HsuidHeader.class).createMarshaller().marshal(HsuidHeaderUtil.getCitizenHsuidHeader(), doc);

        Document medcomDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        JAXBContext.newInstance(dk.medcom.dgws._2006._04.dgws_1_0.Header.class).createMarshaller().marshal(getMedcomHeader(), medcomDoc);

        List<SoapHeader> soapHeaders = new ArrayList<>();
        soapHeaders.add(new SoapHeader(new QName(LogRecordHeaderInterceptor.HSUID_HEADER_NAMESPACE, LogRecordHeaderInterceptor.HSUID_HEADER_NAME, ""), doc, new JAXBDataBinding(HsuidHeader.class)));
        soapHeaders.add(new SoapHeader(new QName(LogRecordHeaderInterceptor.MEDCOM_HEADER_NAMESPACE, LogRecordHeaderInterceptor.MEDCOM_HEADER_NAME, ""), medcomDoc, new JAXBDataBinding(dk.medcom.dgws._2006._04.dgws_1_0.Header.class)));
        setupMocks(soapHeaders);

        // Act
        Object result = subject.generateLogRecordHeader(invocationContext);

        // Assert
        Mockito.verify(httpHeaderContext).addHttpHeader(Mockito.anyString(), Mockito.anyString());
    }

    @Test(expected = IllegalStateException.class)
    public void generateLogRecordHeader_missingHsuidHeader() throws Exception {
        // Arrange
        setupMocks(new ArrayList<>());

        // Act
        subject.generateLogRecordHeader(invocationContext);

        // Assert
    }

    private void setupMocks(List<SoapHeader> soapHeaders) {
        MessageContext messageContext = Mockito.mock(MessageContext.class);
        Mockito.when(messageContext.get(Header.HEADER_LIST)).thenReturn(soapHeaders);

        MockHttpServletRequestImpl httpServletRequest = new MockHttpServletRequestImpl();
        httpServletRequest.addHeader(config.getProperty(SXAConfigKeys.SXA_LOGRECORD_INBOUND_HEADER_NAME), "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><LogRecord xmlns=\"healthcare.loginfo/2017-03-01\"><TraceInfo><RequestTime>2020-10-15T14:45:09.976+02:00</RequestTime><SessionId>00000000-0000-0000-0000-000000000000</SessionId><RequestId>11111111-1111-1111-1111-111111111111</RequestId><Route><ViaSystem id=\"1\">Sundhed.dk</ViaSystem><ViaSystem id=\"2\">DokumentService</ViaSystem><ViaSystem id=\"3\">SvarEksponering PIH Webservice</ViaSystem></Route></TraceInfo><RequestingClient><ClientIdentification ClientIdentificationType=\"CPR nummer\">0987654321</ClientIdentification><ClinicName></ClinicName><ClinicDepartment></ClinicDepartment><UserCprNumber>0987654321</UserCprNumber><UserName></UserName><UserInitials></UserInitials><LookUpType>Borger</LookUpType><TaskType>Opslag Dokumentdelingsservice</TaskType></RequestingClient><Patient><CprNumber>1212121212</CprNumber><Name></Name></Patient></LogRecord>");
        Mockito.when(messageContext.get("HTTP.REQUEST")).thenReturn(httpServletRequest);
        Mockito.when(messageContext.containsKey("HTTP.REQUEST")).thenReturn(true);

        Mockito.when(webServiceContext.getMessageContext()).thenReturn(messageContext);

        Mockito.when(invocationContext.getParameters()).thenReturn(new Object[] { getRequest() });
    }

    private dk.medcom.dgws._2006._04.dgws_1_0.Header getMedcomHeader() {
        dk.medcom.dgws._2006._04.dgws_1_0.Header header = new dk.medcom.dgws._2006._04.dgws_1_0.Header();

        header.setLinking(new Linking());
        header.getLinking().setFlowID("00000000-0000-0000-0000-000000000000");
        header.getLinking().setMessageID("11111111-1111-1111-1111-111111111111");

        return header;
    }

    private RetrieveDocumentSetRequestType getRequest() {
        RetrieveDocumentSetRequestType request = new RetrieveDocumentSetRequestType();

        RetrieveDocumentSetRequestType.DocumentRequest documentRequest = new RetrieveDocumentSetRequestType.DocumentRequest();
        documentRequest.setHomeCommunityId("1.2.3.5.6");
        documentRequest.setRepositoryUniqueId("1.2.3.4.55.66");
        documentRequest.setDocumentUniqueId("1.2.3.4.55.66.12172240491026.20201015115813.20201015115913.0");
        request.getDocumentRequest().add(documentRequest);

        return request;
    }
}