/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentprovider.impl;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocument;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.responses.RetrievedDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.responses.Status;

import dk.nsi.dds.projects.sxa.documentprovider.DocumentRetriever;
import dk.nsi.dds.projects.sxa.documentprovider.DocumentsToRetrieve;
import dk.nsi.dds.projects.sxa.documentprovider.RetrievedDocumentsResult;

@RunWith(MockitoJUnitRunner.class)
public class ProviderRequestHandlerImplTest {
	
	@Mock
	public DocumentRetriever documentRetriever;
	@Mock
	public ProviderConfiguration providerConfiguration;
	@Mock
	public DocumentsToRetrieve documentsToRetrieve;
	@Mock
	public RetrievedDocumentsResult retrievedDocumentsResult;
	
	public ProviderRequestHandler requestHandler;
	
	@Before
	public void setup() {
		requestHandler = new ProviderRequestHandlerImpl(documentRetriever);
	}
	
	@Test
	public void testHandleRequesWithNull() {
		RetrieveDocumentSet in = null;	
		RetrievedDocumentSet out = requestHandler.handleRequest(in);
		
		Assert.assertNotNull(out);
		Assert.assertEquals(0, out.getDocuments().size());
		Assert.assertEquals(0, out.getErrors().size());
		Assert.assertEquals(Status.SUCCESS, out.getStatus());
	}
	
	@Test
	public void testHandleRequestWithEmptyRetrieveDocumentSet() {
		RetrieveDocumentSet in = new RetrieveDocumentSet();	
		RetrievedDocumentSet out = requestHandler.handleRequest(in);
		
		Assert.assertNotNull(out);
		Assert.assertEquals(0, out.getDocuments().size());
		Assert.assertEquals(0, out.getErrors().size());
		Assert.assertEquals(Status.SUCCESS, out.getStatus());
	}
	
	@Test
	public void testHandleRequestWithMock() {
		RetrieveDocumentSet in = new RetrieveDocumentSet();	
		RetrieveDocument doc = new RetrieveDocument();
		in.getDocuments().add(doc);
		
		Mockito.when(documentRetriever.getProviderConfiguration()).thenReturn(providerConfiguration);
		Mockito.when(documentRetriever.retrieveDocuments(Mockito.any())).thenReturn(retrievedDocumentsResult);
		
		RetrievedDocumentSet out = requestHandler.handleRequest(in);
		
		Assert.assertNotNull(out);
		Assert.assertEquals(0, out.getDocuments().size());
		Assert.assertEquals(0, out.getErrors().size());
		Assert.assertEquals(Status.FAILURE, out.getStatus());
	}

}
