package dk.nsi.dds.projects.sxa.documentprovider.interceptor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.enterprise.inject.Produces;
import javax.inject.Inject;
import javax.interceptor.InvocationContext;
import javax.servlet.http.HttpServletRequest;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;

import dk.nsi.dds.projects.sxa.common.configuration.SXAConfigKeys;
import org.jglue.cdiunit.CdiRunner;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;

import dk.nsi.dds.projects.sxa.documentprovider.context.HttpHeaderContext;
import dk.nsi.util.Config;

@RunWith(CdiRunner.class)
public class HttpHeaderReaderInterceptorTest {
    @Inject
    private HttpHeaderReaderInterceptor subject;

    @Produces
    @Mock
    private HttpHeaderContext httpHeaderContext;

    @Produces
    @Mock
    private WebServiceContext webServiceContext;

    @Produces
    @Mock
    private Config config;

    private InvocationContext invocationContext;

    @Before
    public void setup() {
        invocationContext = Mockito.mock(InvocationContext.class);
    }

    @Test
    public void inspectHeaders_canExtractFooHeaderWhenPresent() throws Exception {
        // Arrange
        List<KeyValuePair> headers = new ArrayList<>();
        headers.add(new KeyValuePair("foo", "bar,baz"));
        setupMocks(headers, "foo");

        // Act
        subject.inspectHeaders(invocationContext);

        // Assert
        Mockito.verify(httpHeaderContext).addHttpHeader("foo", "bar,baz");
    }

    @Test
    public void inspectHeaders_allowsConfiguredHeadersToBeMissing() throws Exception {
        // Arrange
        setupMocks(new ArrayList<>(), "foo");

        Map<String, List<String>> expected = new HashMap<>();

        // Act
        subject.inspectHeaders(invocationContext);

        // Assert
        Mockito.verify(httpHeaderContext, Mockito.times(0)).addHttpHeader(Mockito.anyString(), Mockito.anyString());
    }

    @Test
    public void inspectHeaders_ignoresNonConfiguredHeaders() throws Exception {
        // Arrange
        List<KeyValuePair> headers = new ArrayList<>();
        headers.add(new KeyValuePair("foo", "bar,baz"));
        setupMocks(headers, "qux");

        Map<String, List<String>> expected = new HashMap<>();

        // Act
        subject.inspectHeaders(invocationContext);

        // Assert
        Mockito.verify(httpHeaderContext, Mockito.times(0)).addHttpHeader(Mockito.anyString(), Mockito.anyString());
    }

    private void setupMocks(List<KeyValuePair> headers, String headerProperty) {
        Mockito.when(config.getProperty(SXAConfigKeys.SXA_HEADERS)).thenReturn(headerProperty);

        HttpServletRequest request = Mockito.mock(HttpServletRequest.class);
        for(KeyValuePair kvp : headers) {
            Mockito.when(request.getHeader(kvp.key)).thenReturn(kvp.value);
        }

        MessageContext messageContext = Mockito.mock(MessageContext.class);
        Mockito.when(messageContext.containsKey(Mockito.anyString())).thenReturn(true);
        Mockito.when(messageContext.get(Mockito.anyString())).thenReturn(request);

        Mockito.when(webServiceContext.getMessageContext()).thenReturn(messageContext);
    }

    private class KeyValuePair {
        KeyValuePair(String key, String value) {
            this.key = key;
            this.value = value;
        }

        String key;
        String value;
    }
}
