package dk.nsi.dds.projects.sxa.documentprovider.interceptor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.enterprise.inject.Produces;

import org.apache.cxf.message.Message;
import org.apache.cxf.message.MessageImpl;
import org.jglue.cdiunit.CdiRunner;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;

import dk.nsi.dds.projects.sxa.documentprovider.context.HttpHeaderContext;

@RunWith(CdiRunner.class)
public class HttpMessageInterceptorTest {
    //@Inject
    private HttpMessageInterceptor subject;

    @Produces
    @Mock
    private HttpHeaderContext httpHeaderContext;

    @Before
    public void setup() {
        subject = new HttpMessageInterceptor(httpHeaderContext);
    }

    @Test
    public void handleMessage_NoExistingHeaders_HeadersAreInitialized() {
        // Arrange
        Message message = Mockito.mock(Message.class);

        // Act
        subject.handleMessage(message);

        // Assert
        Mockito.verify(message).put(Mockito.anyString(), Mockito.anyMap());
    }

    @Test
    public void handleMessage_nonExistingFooHeaderIsTransferred() {
        // Arrange
        Map<String, List<String>> protocolHeaders = new HashMap<>();
        Message message = new MessageImpl();
        message.put(Message.PROTOCOL_HEADERS, protocolHeaders);

        Map<String, List<String>> headers = new HashMap<>();
        headers.put("foo", new ArrayList<>());
        headers.get("foo").add("bar");
        Mockito.when(httpHeaderContext.getHttpHeaders()).thenReturn(headers);

        // Act
        subject.handleMessage(message);

        // Assert
        Assert.assertTrue("Expected foo header to be populated!", protocolHeaders.containsKey("foo"));
        Assert.assertTrue(protocolHeaders.get("foo").contains("bar"));
    }

    @Test
    public void handleMessage_nonExistingBarHeaderIsTransferred() {
        // Arrange
        Map<String, List<String>> protocolHeaders = new HashMap<>();
        Message message = new MessageImpl();
        message.put(Message.PROTOCOL_HEADERS, protocolHeaders);

        Map<String, List<String>> headers = new HashMap<>();
        headers.put("bar", new ArrayList<>());
        headers.get("bar").add("baz");
        Mockito.when(httpHeaderContext.getHttpHeaders()).thenReturn(headers);

        // Act
        subject.handleMessage(message);

        // Assert
        Assert.assertTrue("Expected bar header to be populated!", protocolHeaders.containsKey("bar"));
        Assert.assertTrue(protocolHeaders.get("bar").contains("baz"));
    }

    @Test
    public void handleMessage_existingBazHeaderIsRetained() {
        // Arrange
        Map<String, List<String>> protocolHeaders = new HashMap<>();
        protocolHeaders.put("baz", new ArrayList<>());
        protocolHeaders.get("baz").add("qux");
        Message message = new MessageImpl();
        message.put(Message.PROTOCOL_HEADERS, protocolHeaders);

        Map<String, List<String>> headers = new HashMap<>();
        headers.put("baz", new ArrayList<>());
        headers.get("baz").add("quux");
        Mockito.when(httpHeaderContext.getHttpHeaders()).thenReturn(headers);

        // Act
        subject.handleMessage(message);

        // Assert
        Assert.assertTrue("Expected baz header to be populated!", protocolHeaders.containsKey("baz"));
        Assert.assertTrue(protocolHeaders.get("baz").contains("qux"));
        Assert.assertTrue(protocolHeaders.get("baz").contains("quux"));
    }
}
