/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in the
 * Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.documentprovider.util;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.Assert;
import org.junit.Test;

import com.cgi.svareksponering.Speciale;

import dk.nsi.dds.projects.sxa.common.TestDataHelper;
import dk.nsi.dds.projects.sxa.common.configuration.SXAMappedValue;

public class SpecialtyMapperTest {

  private static TestDataHelper configTestData = new TestDataHelper();
  private static SXAMappedValue microbiologyCode = configTestData.practiceSettingCodeForMicrobiology;
  private static SXAMappedValue biochemistryCode = configTestData.practiceSettingCodeForBiochemistry;
  private static List<SXAMappedValue> validPracticeSettingCodesList = configTestData.getPracticeSettingCodes();

  @Test
  public void emptyPracticeSettingCodeListMapsToAll() {
    Speciale[] allSpecialer = Stream.of(Speciale.KLINISK_BIOKEMI, Speciale.MIKROBIOLOGI, Speciale.PATOLOGI).toArray(size -> new Speciale[size]);
    assertSpecialtyArrayContentEquals(allSpecialer, SpecialtyMapper.mapToSpecialty(Collections.emptyList(), validPracticeSettingCodesList));
  }

  @Test
  public void microbiologyPracticeSettingCodeListMapsToMicrobiology() {
    Speciale[] expectedSpecialer = Stream.of(Speciale.MIKROBIOLOGI).toArray(size -> new Speciale[size]);
    assertSpecialtyArrayContentEquals(expectedSpecialer, SpecialtyMapper.mapToSpecialty(TestDataHelper.asCodeList(microbiologyCode), validPracticeSettingCodesList));
  }

  @Test
  public void multiplePracticeSettingCodeListMapsToCorresponding() {
    Speciale[] expectedSpecialer = Stream.of(Speciale.MIKROBIOLOGI, Speciale.KLINISK_BIOKEMI).toArray(size -> new Speciale[size]);
    assertSpecialtyArrayContentEquals(expectedSpecialer, SpecialtyMapper.mapToSpecialty(TestDataHelper.asCodeList(microbiologyCode, biochemistryCode), validPracticeSettingCodesList));
  }
  @Test(expected=NoMatchingPracticeSettingCodesException.class)
  public void nonMatchingSettingCodesThrowsException() {
    SpecialtyMapper.mapToSpecialty(TestDataHelper.asCodeList(microbiologyCode, biochemistryCode), Arrays.asList(configTestData.practiceSettingInvalidCodeForPathology));
  }

  private void assertSpecialtyArrayContentEquals(Speciale[] expected, Speciale[] actual) {
    List<Speciale> expectedList = Stream.of(expected).sorted().collect(Collectors.toList());
    List<Speciale> actualList = Stream.of(actual).sorted().collect(Collectors.toList());
    Assert.assertArrayEquals(expectedList.toArray(), actualList.toArray());
  }
  
  
}
