/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common;

import java.io.File;

import com.thoughtworks.xstream.XStream;

import dk.nsi.dds.projects.sxa.common.configuration.SXAMappedValue;
import dk.nsi.dds.projects.sxa.common.util.configloading.FileModifiedWatcher;
import dk.nsi.dds.projects.sxa.common.util.xmlhandling.XStreamFileException;
import dk.nsi.dds.projects.sxa.common.util.xmlhandling.XStreamFileHandler;

public class PracticeSettingCodesHandler {

	private XStreamFileHandler xStreamCodeHandler;
	private FileModifiedWatcher codeFile;
	private PracticeSettingCodesMappingConfiguration mappingConfigurationConfiguration;

	public PracticeSettingCodesHandler(File codeFile) {
		super();
		this.codeFile = new FileModifiedWatcher(codeFile);
		xStreamCodeHandler = new XStreamFileHandler(codeFile.getPath(), createXStream());
	}

	private XStream createXStream() {
		XStream xstream = new XStream();
		xstream.alias("filter", PracticeSettingCodesConfiguration.class);
		xstream.alias("mappedCodedValue", SXAMappedValue.class);
		return xstream;
	}

	public PracticeSettingCodesMappingConfiguration readConfiguration() throws XStreamFileException {
		if (codeFile.isModified()) {
			synchronized (this) {
				if (codeFile.isModified()) {
					PracticeSettingCodesConfiguration conf = xStreamCodeHandler.readXmlFile(PracticeSettingCodesConfiguration.class);
					PracticeSettingCodesMappingConfiguration configuration = new PracticeSettingCodesMappingConfiguration();
					configuration.setPracticeSettingCodesConfiguration(conf);
					mappingConfigurationConfiguration = configuration;
					codeFile.reload();
				}
			}
		}
		return mappingConfigurationConfiguration;
	}
}
