/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in the
 * Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common;

import java.util.List;

import org.joda.time.DateTime;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Code;

import dk.nsi.dds.projects.sxa.common.configuration.SXAMappedValue;

/**
 * Details from the query resulting in the request for the document.
 *
 */
public class QueryDetails {
	private String patientId;
	private DateTime from;
	private DateTime to;
	private List<Code> practiceSettingCodes;
	private List<SXAMappedValue> mappedPracticeSettingCodesList;

	public QueryDetails(String id, List<SXAMappedValue> validPracticeSettingCodesList) throws InvalidSXAQueryParameters {
		try {
			this.mappedPracticeSettingCodesList = validPracticeSettingCodesList;
			DocumentIdEncoderDecoder decoder = new DocumentIdEncoderDecoder(id, validPracticeSettingCodesList);
			this.patientId = decoder.getPatientId();
			this.from = decoder.getFrom();
			this.to = decoder.getTo();
			if (this.to != null && this.from != null) {
				if (from.isAfter(to)) {
					throw new InvalidSXAQueryParameters("'to' må ikke være før 'from'");
				}
			}
			this.practiceSettingCodes = decoder.getPracticeSettingCodes();
		} catch (IllegalArgumentException ex) {
			throw new InvalidSXAQueryParameters("Document-id er ikke korrekt kodet");
		}
	}

	public String getPatientId() {
		return patientId;
	}

	public DateTime getFrom() {
		return from;
	}

	public DateTime getTo() {
		return to;
	}

	public List<Code> getPracticeSettingCodes() {
		return practiceSettingCodes;
	}

	public List<SXAMappedValue> getMappedPracticeSettingCodesList() {
		return mappedPracticeSettingCodesList;
	}
}
