/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common;

import org.apache.log4j.Logger;
import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;

import dk.nsi.dds.projects.sxa.common.core.interval.UnboundableInterval;

public class QueryIntervalCreator {
	private static final Logger logger = Logger.getLogger(QueryIntervalCreator.class);
	private static final DateTime DEFAULT_START_TIME = new DateTime(1900, 1, 1, 0, 0, DateTimeZone.UTC);

	public static UnboundableInterval createQueryInterval(QueryDetails query) {
		return createQueryInterval(query.getPatientId(), query.getFrom(), query.getTo());
	}

	public static UnboundableInterval createQueryInterval(String patientId, DateTime start, DateTime end) {
		// Be careful not to provide null ReadableInstants to Interval as they
		// are interpreted as "now"
		if (start == null && end == null) {
			return deriveIntervalFromPatientDateOfBirth(patientId);
		} else {
			return new UnboundableInterval(start, end);
		}
	}

	/**
	 * Creates an interval where the start is determined from the date of birth present in the patient id. The interval returned has no end.
	 */
	public static UnboundableInterval deriveIntervalFromPatientDateOfBirth(String patientId) {
		try {
			return new UnboundableInterval(PersonIdInfoDerivation.deriveDateOfBirth(patientId), null);
		} catch (PatientIdDerivationException e) {
			logger.debug(e);
			return new UnboundableInterval(DEFAULT_START_TIME, null);
		}
	}
}
