/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common.configuration;

import org.openehealth.ipf.commons.ihe.xds.core.metadata.Name;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Person;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.XcnName;

import ca.uhn.hl7v2.model.v25.datatype.XCN;

public class XPerson {
	private XIdentifiable id;
	private String givenName, middleName, surname ,suffix ,prefix, degree;
	
	public XPerson(Person p) {
		id = new XIdentifiable(p.getId());
		givenName = p.getName().getGivenName();
		middleName = p.getName().getSecondAndFurtherGivenNames();
		surname = p.getName().getFamilyName();
		suffix = p.getName().getSuffix();
		prefix = p.getName().getPrefix(); 
		degree = p.getName().getDegree();
	}
	
	public Person toPerson(){
		Name<XCN> name = new XcnName(surname, givenName, middleName, suffix, prefix, degree);
		return new Person(id.toIdentifiable(), name);
	}
	
	public static Person getPerson(XPerson xp) {
		return xp != null ? xp.toPerson() : null;
	}
	
	public static XPerson getXPerson(Person p) {
		return p != null ?new XPerson(p) : null;
	}
}
