/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common.core.query;

import java.util.regex.Pattern;

/**
 * Formats query parameters according to IHE TF-2a section 3.18.4.1.2.3
 *
 */
public class QueryParameterFormatter {

	// Matches digits in the following forms (1 used as example): [1; 1.0; -1; -1.0; 1,0; -1,0]
	private static final Pattern IS_NUMBER_PATTERN = Pattern.compile("^-?\\d+\\,?\\.?\\d*$");

	public static String createSingleValuedQueryParameter(String value) {
		if (IS_NUMBER_PATTERN.matcher(value).matches()) {
			return value;
		} else if (value != null && value.contains("'")) {
			return value.replaceAll("'", "''");
		}
		return "\'" + value + "\'";
	}

	public static String createMultiValuedQueryParameter(String value) {
		return "(" + createSingleValuedQueryParameter(value) + ")";
	}

}