/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common.id;

import java.math.BigInteger;
import java.util.UUID;

public class IdConverter {
	private static final String URN_UUID_PREFIX = "urn:uuid:";
	private static final String UUID_AS_OID_ROOT = "2.25.";

	public static String convertUuidToOid(UUID uuid) {
		return UUID_AS_OID_ROOT + convertUuidToOidNoPrefix(uuid);
	}

	public static String convertUuidToOid(String uuid) {
		return UUID_AS_OID_ROOT + convertUuidToOidNoPrefix(uuid);
	}

	public static String convertUuidToOidNoPrefix(UUID uuid) {
		String uuidAsHexString = removeHyphenAndUrnPrefix(uuid.toString());
		return new BigInteger(uuidAsHexString, 16).toString();
	}

	public static String convertUuidToOidNoPrefix(String uuid) {
		String uuidAsHexString = removeHyphenAndUrnPrefix(uuid.toString());
		return new BigInteger(uuidAsHexString, 16).toString();
	}

	public static String removeHyphenAndUrnPrefix(String uuid) {
		return uuid.replace("-", "").replace(URN_UUID_PREFIX, "");
	}

	public static String addHyphenAndUrnPrefix(String uuidWithoutHyphensAndUrn) {
		String withHyphens = removeHyphenAndUrnPrefix(uuidWithoutHyphensAndUrn).replaceAll("(\\w{8})(\\w{4})(\\w{4})(\\w{4})(\\w{12})", "$1-$2-$3-$4-$5");
		return URN_UUID_PREFIX + withHyphens;
	}

	public static String convertOidToUuid(String oid) {
		return addHyphenAndUrnPrefix(convertOidToUuidNoPrefix(oid));
	}

	public static String convertOidToUuidNoPrefix(String oid) {
		if (oid != null) {
			String toConvert = oid;
			if (oid.startsWith(UUID_AS_OID_ROOT)) {
				toConvert = oid.replace(UUID_AS_OID_ROOT, "");
			}
			return new BigInteger(toConvert).toString(16);
		}
		return null;
	}
}
