/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common.nationalprofile;

import org.openehealth.ipf.commons.ihe.xds.core.metadata.AssigningAuthority;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Identifiable;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Name;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Organization;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Person;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.XcnName;

import ca.uhn.hl7v2.model.v25.datatype.XCN;

/**
 * Factory capable of producing values and other constructs making use of code systems used in the (Danish) National XDS Content Profile for Document Sharing.
 */
public class NationalProfileValueFactory {
	public Identifiable createPatientIdUsingCpr(String personCprNumber) {
		return new Identifiable(personCprNumber, new AssigningAuthority(NationalProfileConstants.ASSIGNINGAUTHORITY_OID_DANISH_PERSON_CIVIL_REGISTRATION_ID));
	}

	public Organization createOrganisationUsingSORCode(String organisationName, String organisationSorCode) {
		return new Organization(organisationName, organisationSorCode, new AssigningAuthority(NationalProfileConstants.ASSIGNINGAUTHORITY_OID_DANISH_ORGANISATION_SOR_ID));
	}

	@Deprecated
	public Organization createOrganisationUsingSHAKCode(String organisationName, String organisationShakCode) {
		return new Organization(organisationName, organisationShakCode, new AssigningAuthority("TODO"));
	}

	/**
	 * Creates a value with a General Practitioner Number (in Danish: Ydernummer)
	 * 
	 * @return
	 */
	public Organization createOrganisationUsingGPNumber(String organisationName, String organisationGPNumber) {
		return new Organization(organisationName, organisationGPNumber, new AssigningAuthority(NationalProfileConstants.ASSIGNINGAUTHORITY_OID_DANISH_ORGANSIATION_PROVIDER_REGISTER_ID));
	}

	public Person createPersonUsingCpr(String cprNumber, String givenName, String middleName, String surname) {
		Identifiable id = new Identifiable(cprNumber, new AssigningAuthority(NationalProfileConstants.ASSIGNINGAUTHORITY_OID_DANISH_PERSON_CIVIL_REGISTRATION_ID));
		Name<XCN> name = new XcnName(surname, givenName, middleName, null, null, null);
		return new Person(id, name);
	}

	/**
	 * Creates a person representation
	 * 
	 * @param cprNumber  id given in the Danish Central Person Register
	 * @param givenName
	 * @param middleName
	 * @param surname
	 * @param prefix     title, for instance Dr.
	 * @param suffix     addendum to name, for instance Jr.
	 * @param degree     academic degree, for instance Med.
	 * @return
	 */
	public Person createPersonUsingCpr(String cprNumber, String givenName, String middleName, String surname, String prefix, String suffix, String degree) {
		Identifiable id = new Identifiable(cprNumber, new AssigningAuthority(NationalProfileConstants.ASSIGNINGAUTHORITY_OID_DANISH_PERSON_CIVIL_REGISTRATION_ID));
		Name<XCN> name = new XcnName(surname, givenName, middleName, prefix, suffix, degree);
		return new Person(id, name);
	}
}
