/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common.util;

import java.util.Collections;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Code;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Identifiable;

import dk.nsi.dds.projects.sxa.common.DocumentIdEncoderDecoder;
import dk.nsi.dds.projects.sxa.common.configuration.SXAMappedValue;

public class DocumentUniqueIdUtil {
	
	private final static String SEPARATOR = ".";

	public static String createUniqueId(String uniqueIdRoot, Identifiable patientId, DateTime serviceStartDateTime, DateTime serviceStopDateTime, List<Code> practiceSettingCodes, List<SXAMappedValue> validPracticeSettingCodes) {
		if(patientId == null || StringUtils.isEmpty(patientId.getId())) {
			throw new IllegalArgumentException("Missing patientId");
		}
		String encodedId = new DocumentIdEncoderDecoder(patientId.getId(), serviceStartDateTime, serviceStopDateTime, practiceSettingCodes, validPracticeSettingCodes).encode();
		return uniqueIdRoot + SEPARATOR + encodedId;
	}

	public static String getUniqueIdWithoutRoot(String root, String documentUniqueId) {
		String prefix = root + SEPARATOR;
	    if (!documentUniqueId.startsWith(prefix)) {
	      throw new IllegalArgumentException("id does not have the provided root");
	    }
	    return documentUniqueId.substring(prefix.length());		
	}
	
	public static void main(String[] args) {
		
		System.out.println(createUniqueId("1.2.3.4.55.66", new Identifiable("121212121212", null), DateTime.now(), DateTime.now().plusDays(5), Collections.emptyList(), Collections.emptyList()));
		//System.out.println(getUniqueIdWithoutRoot("1.2.3.4.55.66", "1.2.3.4.55.66^urn:uuid:5478c74f-acb1-4e4d-0c65-a525252ea1767832288a22"));
	}

}
