/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common.util;

import java.util.ArrayList;
import java.util.List;

import org.joda.time.Interval;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AvailabilityStatus;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Code;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.DocumentEntryType;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Hl7v2Based;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Person;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.FindDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.QueryList;

import dk.nsi.dds.projects.sxa.common.configuration.SXAMappedValue;

public class QueryUtil {

	public static void addClassCode(FindDocumentsQuery query, SXAMappedValue classCode) {
		//validate(query, QueryParameter.DOC_ENTRY_CLASS_CODE);
		
		List<Code> classCodes = query.getClassCodes();
		if(classCodes == null) {
			classCodes = new ArrayList<>();
			query.setClassCodes(classCodes);
		}
		classCodes.add(ConverterUtil.convert(classCode));
	}

	public static void addTypeCode(FindDocumentsQuery query, SXAMappedValue typeCode) {
		List<Code> typeCodes = query.getTypeCodes();
		if(typeCodes == null) {
			typeCodes = new ArrayList<>();
			query.setTypeCodes(typeCodes);
		}
		typeCodes.add(ConverterUtil.convert(typeCode));	
	}

	public static void addConfidentialityCode(FindDocumentsQuery query, SXAMappedValue confidentialityCode) {
		QueryList<Code> confidentialityCodes = query.getConfidentialityCodes();
		if(confidentialityCodes == null) {
			confidentialityCodes = new QueryList<>();
			query.setConfidentialityCodes(confidentialityCodes);
		}
		List<List<Code>> outerList = confidentialityCodes.getOuterList();
		List<Code> confidentialityCodeList = outerList.isEmpty() ? newList(outerList) : outerList.get(0);
		confidentialityCodeList.add(ConverterUtil.convert(confidentialityCode));
	}
	
	private static List<Code> newList(List<List<Code>> outerList){
		List<Code> result = new ArrayList<>();
		outerList.add(result);
		return result;
	}

	public static void addEventCode(FindDocumentsQuery query, SXAMappedValue eventCode) {
		QueryList<Code> eventCodes = query.getEventCodes();
		if(eventCodes == null) {
			eventCodes = new QueryList<>();
			query.setEventCodes(eventCodes);
		}
		List<List<Code>> outerList = eventCodes.getOuterList();
		List<Code> eventCodeList = outerList.isEmpty() ? newList(outerList) : outerList.get(0);
		eventCodeList.add(ConverterUtil.convert(eventCode));
	}

	public static void addFormatCode(FindDocumentsQuery query, SXAMappedValue formatCode) {
		List<Code> formatCodes = query.getFormatCodes();
		if(formatCodes == null) {
			formatCodes = new ArrayList<>();
			query.setFormatCodes(formatCodes);
		}
		formatCodes.add(ConverterUtil.convert(formatCode));	
	}

	public static void addHealtcareFacilityTypeCode(FindDocumentsQuery query, SXAMappedValue healthcareFacilityTypeCode) {
		List<Code> healthcareFacilityTypeCodes = query.getHealthcareFacilityTypeCodes();
		if(healthcareFacilityTypeCodes == null) {
			healthcareFacilityTypeCodes = new ArrayList<>();
			query.setHealthcareFacilityTypeCodes(healthcareFacilityTypeCodes);
		}
		healthcareFacilityTypeCodes.add(ConverterUtil.convert(healthcareFacilityTypeCode));
		
		//validate(query, QueryParameter.DOC_ENTRY_HEALTHCARE_FACILITY_TYPE_CODE);
	}

	public static void addPracticeSettingCode(FindDocumentsQuery query, SXAMappedValue practiceSettingCode) {
		List<Code> practiceSettingCodes = query.getPracticeSettingCodes();
		if(practiceSettingCodes == null) {
			practiceSettingCodes = new ArrayList<>();
			query.setPracticeSettingCodes(practiceSettingCodes);
		}
		practiceSettingCodes.add(ConverterUtil.convert(practiceSettingCode));	
	}

	public static void addDocumentEntryType(FindDocumentsQuery query, DocumentEntryType documentEntryType) {
		List<DocumentEntryType> documentEntryTypes = query.getDocumentEntryTypes();
		if(documentEntryTypes == null) {
			documentEntryTypes = new ArrayList<>();
			query.setDocumentEntryTypes(documentEntryTypes);
		}
		documentEntryTypes.add(documentEntryType);		
	}

	public static void addStatus(FindDocumentsQuery query, AvailabilityStatus availabilityStatus) {
		List<AvailabilityStatus> status = query.getStatus();
		if(status == null) {
			status = new ArrayList<>();
			query.setStatus(status);
		}
		status.add(availabilityStatus);		
	}

	public static void setCreationTime(FindDocumentsQuery query, Interval creationTime) {
		if(creationTime == null) {
			return;
		}
		
		query.getCreationTime().setFrom(creationTime.getStart());
		query.getCreationTime().setTo(creationTime.getEnd());	
	}

	public static void addAuthorPerson(FindDocumentsQuery query, Person authorPerson) {
		List<String> authorPersons = query.getAuthorPersons();
		if(authorPersons == null) {
			authorPersons = new ArrayList<>();
			query.setAuthorPersons(authorPersons);
		}
		
		authorPersons.add(Hl7v2Based.render(authorPerson));			
	}
	
//	private static void validate(FindDocumentsQuery query, final QueryParameter queryParam) {
//		CodeValidation validation = new CodeValidation(queryParam);	
//		EbXMLAdhocQueryRequest ebXML = new EbXMLFactory30().createAdhocQueryRequest();
//		new FindDocumentsQueryTransformer<FindDocumentsQuery>().toEbXML(query, ebXML);
//		validation.validate(ebXML);
//	}

}
