/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common.util.xmlhandling;

import java.io.Closeable;
import java.io.Flushable;
import java.io.IOException;

import com.thoughtworks.xstream.XStream;

public class XStreamHandler {
	protected XStream xstream;

	protected static void checkNull(Object obj, String errorMessage) throws XStreamFileException {
		if (obj == null) {
			throw new XStreamFileException(errorMessage);
		}
	}

	protected static void checkEmptyString(String xml) throws XStreamFileException {
		if (xml == null) {
			throw new XStreamFileException("Xml string from XStream is null");
		}
		if (xml.isEmpty()) {
			throw new XStreamFileException("Xml string from XStream is empty");
		}
	}

	protected <T> T castObjectToType(Class<T> clazz, Object obj) throws XStreamFileException {
		if (obj == null) {
			throw new XStreamFileException("Object from XStream is null");
		}
		if (obj.getClass() != clazz) {
			throw new XStreamFileException(String.format("Deserialized object is not same type as expected type: expected: %s, actually: %s", clazz, obj.getClass()));
		}
		return clazz.cast(obj);
	}

	protected static void close(Closeable closeable) {
		if (closeable != null) {
			if (closeable instanceof Flushable) {
				flush((Flushable) closeable);
			}
			try {
				closeable.close();
			} catch (IOException e) {
				// Ignore
			}
		}
	}

	private static void flush(Flushable flushable) {
		if (flushable != null) {
			try {
				flushable.flush();
			} catch (IOException e) {
				// Ignore
			}
		}
	}
}
