package dk.nsi.dds.projects.sxa.common;

import dk.nsi.hsuid.*;
import dk.nsi.hsuid._2016._08.hsuid_1_1.HsuidHeader;

import java.util.ArrayList;
import java.util.List;

public class HsuidHeaderUtil {
    public static HsuidHeader getCitizenHsuidHeader() {
        return getValidCitizenHsuidHeader("issuer", "0123456789", "System owner", "System name", "0.0", "Responsible org", false);
    }

    public static HsuidHeader getBehalfOfCitizenHsuidHeader() {
        return getValidBehalfOfCitizenHsuidHeader("issuer", "0123456789", "9876543210", "System owner", "System name", "0.0", "Responsible org");
    }

    public static HsuidHeader getConsentOverrideCitizenHsuidHeader() {
        return getValidCitizenHsuidHeader("issuer", "0123456789", "System owner", "System name", "0.0", "Responsible org", true);
    }

    public static HsuidHeader getSksHealthcareProfessionalHsuidHeader(String organisationIdentifier) {
        return getValidHealthcareProfessionalHsuidHeader("issuer", organisationIdentifier, OrganisationIdentifierAttribute.LegalFormatNames.SHAK, "System owner", "System name", "0.0", "Responsible org", null);
    }

    public static HsuidHeader getSksHealthcareProfessionalHsuidHeaderWithResponsibleAuthorizationCode(String organisationIdentifier) {
        return getValidHealthcareProfessionalHsuidHeader("issuer", organisationIdentifier, OrganisationIdentifierAttribute.LegalFormatNames.SHAK, "System owner", "System name", "0.0", "Responsible org", "445566");
    }

    public static HsuidHeader getSorHealthcareProfessionalHsuidHeader(String organisationIdentifier) {
        return getValidHealthcareProfessionalHsuidHeader("issuer", organisationIdentifier, OrganisationIdentifierAttribute.LegalFormatNames.SOR, "System owner", "System name", "0.0", "Responsible org", null);
    }

    public static HsuidHeader getYderHealthcareProfessionalHsuidHeader(String organisationIdentifier) {
        return getValidHealthcareProfessionalHsuidHeader("issuer", organisationIdentifier, OrganisationIdentifierAttribute.LegalFormatNames.GP_NUMBER, "System owner", "System name", "0.0", "Responsible org", null);
    }

    private static HsuidHeader getValidCitizenHsuidHeader(String issuer, String cpr, String systemOwner, String systemName, String systemVersion, String responsibleOrg, boolean consentOverride) {
        List<Attribute> attributes = new ArrayList<Attribute>(6);
        addBasicCitizenAttributes(true, cpr, systemOwner, systemName, systemVersion, responsibleOrg, attributes);
        if(consentOverride) {
            attributes.add(new ConsentOverrideAttribute(true));
        }
        return HealthcareServiceUserIdentificationHeaderUtil.createHealthcareServiceUserIdentificationHeader(issuer, attributes);
    }

    private static HsuidHeader getValidBehalfOfCitizenHsuidHeader(String issuer, String cpr, String behalfOfCpr, String systemOwner, String systemName, String systemVersion, String responsibleOrg) {
        List<Attribute> attributes = new ArrayList<Attribute>(6);
        attributes.add(new ResponsibleUserCivilRegistrationNumberAttribute(behalfOfCpr));
        attributes.add(new CitizenUserRelationAttribute("nsi:ChildCustodyHolder"));
        addBasicCitizenAttributes(true, cpr, systemOwner, systemName, systemVersion, responsibleOrg, attributes);
        return HealthcareServiceUserIdentificationHeaderUtil.createHealthcareServiceUserIdentificationHeader(issuer, attributes);
    }

    private static HsuidHeader getValidHealthcareProfessionalHsuidHeader(String issuer, String organisationIdentifier, OrganisationIdentifierAttribute.LegalFormatNames organisationType, String systemOwner, String systemName, String systemVersion, String responsibleOrg, String responsibleAuthorizationCode) {
        List<Attribute> attributes = new ArrayList<Attribute>(6);
        addBasicHealthcareProfessionalAttributes(false, organisationIdentifier, organisationType, systemOwner, systemName, systemVersion, responsibleOrg, attributes);
        if(responsibleAuthorizationCode != null) {
            attributes.add(new ResponsibleUserAuthorizationCodeAttribute(responsibleAuthorizationCode));
        }
        return HealthcareServiceUserIdentificationHeaderUtil.createHealthcareServiceUserIdentificationHeader(issuer, attributes);
    }

    private static void addBasicCitizenAttributes(boolean isCitizen, String cpr, String systemOwner, String systemName, String systemVersion, String responsibleOrg, List<Attribute> attributes) {
        attributes.add(new UserTypeAttribute(isCitizen));
        attributes.add(new ActingUserCivilRegistrationNumberAttribute("0987654321"));
        attributes.add(new CitizenCivilRegistrationNumberAttribute(cpr));
        attributes.add(new SystemVendorNameAttribute(systemOwner));
        attributes.add(new SystemNameAttribute(systemName));
        attributes.add(new SystemVersionAttribute(systemVersion));
        attributes.add(new OperationsOrganisationNameAttribute(responsibleOrg));
    }

    private static void addBasicHealthcareProfessionalAttributes(boolean isCitizen, String organisationIdentifier, OrganisationIdentifierAttribute.LegalFormatNames organisationType, String systemOwner, String systemName, String systemVersion, String responsibleOrg, List<Attribute> attributes) {
        attributes.add(new UserTypeAttribute(isCitizen));
        attributes.add(new ActingUserCivilRegistrationNumberAttribute("0987654321"));
        attributes.add(new OrganisationIdentifierAttribute(organisationIdentifier, organisationType));
        attributes.add(new SystemVendorNameAttribute(systemOwner));
        attributes.add(new SystemNameAttribute(systemName));
        attributes.add(new SystemVersionAttribute(systemVersion));
        attributes.add(new OperationsOrganisationNameAttribute(responsibleOrg));
    }
}
