/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in the
 * Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.junit.BeforeClass;
import org.junit.Test;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Code;

import dk.nsi.dds.projects.sxa.common.configuration.SXAMappedValue;
import dk.nsi.dds.projects.sxa.common.util.ConverterUtil;

import org.junit.Assert;

public class PracticeSettingCodesEncoderDecoderTest {
	private static PracticeSettingCodesEncoderDecoder encoder;
	private static TestDataHelper testDataHelper = new TestDataHelper();

	@BeforeClass
	public static void setup() {
		List<SXAMappedValue> validCodes = Arrays.asList(testDataHelper.practiceSettingCodeForMicrobiology, testDataHelper.practiceSettingCodeForBiochemistry,
				testDataHelper.practiceSettingCodeForPathology);
		encoder = new PracticeSettingCodesEncoderDecoder(validCodes);
	}

	@Test
	public void encodeWithTwoValidCodes() {
		// Arrange
		List<Code> testList = Arrays.asList(ConverterUtil.convert(testDataHelper.practiceSettingCodeForMicrobiology), ConverterUtil.convert(testDataHelper.practiceSettingCodeForBiochemistry));
		// Act
		int encodedValue = encoder.encode(testList);
		// Assert
		Assert.assertEquals(3, encodedValue);
	}

	@Test
	public void encodeWithEmptyList() {
		// Arrange
		List<Code> testList = Arrays.asList();
		// Act
		int encodedValue = encoder.encode(testList);
		// Assert
		Assert.assertEquals(0, encodedValue);
	}

	@Test
	public void encodeWithInvalidPracticeSettingCodeInList() {
		// Arrange
		List<Code> testList = Arrays.asList(ConverterUtil.convert(testDataHelper.practiceSettingCodeForMicrobiology), ConverterUtil.convert(testDataHelper.practiceSettingCodeNonSxa));
		// Act
		int encodedValue = encoder.encode(testList);
		// Assert
		Assert.assertEquals(1, encodedValue);
	}

	/**
	 * Using more than 32 codes throws an exception on constructor call
	 */
	@Test(expected = IllegalArgumentException.class)
	public void constructorWithTooManyValidCodes() {
		// Arrange
		List<Code> testList = new ArrayList<>();
		for (int i = 0; i < 33; i++) {
			testList.add(ConverterUtil.convert(testDataHelper.practiceSettingCodeForMicrobiology));
		}
		// Act
		encoder.encode(testList);
	}

	@Test
	public void decodeValidValue() {
		// Arrange
		int testValue = 7;
		List<Code> expectedList = Arrays.asList(ConverterUtil.convert(testDataHelper.practiceSettingCodeForMicrobiology), 
				ConverterUtil.convert(testDataHelper.practiceSettingCodeForBiochemistry),
				ConverterUtil.convert(testDataHelper.practiceSettingCodeForPathology));
		// Act
		List<Code> decodedList = encoder.decode(testValue);
		// Assert
		Assert.assertEquals(expectedList, decodedList);
	}

	@Test(expected = IllegalArgumentException.class)
	public void decodeInvalidNegativeValue() {
		// Arrange
		int negativeTestValue = -1;
		// Act
		encoder.decode(negativeTestValue);
	}

	@Test(expected = IllegalArgumentException.class)
	public void decodeToLargeAValue() {
		// Arrange
		int toLargeTestValue = 8;
		// Act
		encoder.decode(toLargeTestValue);
	}
}
