/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common;

import org.joda.time.DateTime;
import org.junit.Test;

import dk.nsi.dds.projects.sxa.common.core.interval.UnboundableInterval;
import org.junit.Assert;

public class QueryIntervalCreatorTest {
	
    	private DateTime now = new DateTime();
    	private DateTime start = now.minusDays(5);
    	private DateTime end = now.minusDays(1);
    	private String patientId = "2512484916";
    
	@Test
	public void createQueryInterval_PatientIdAboveCurrentYearTest() {
		// Arrange
		DateTime currentDate = new DateTime();
		String currentYearTekst = currentDate.getYear() + 1 + "";
		String currentYearDecade = currentYearTekst.substring(currentYearTekst.length() - 2, currentYearTekst.length());
		String expectedYearTekst = currentYearTekst.substring(0, currentYearTekst.length() - 2) + currentYearDecade;
		
		// Act
		UnboundableInterval interval = QueryIntervalCreator.deriveIntervalFromPatientDateOfBirth("0101" + currentYearDecade + "1234");
		
		// Assert
		Assert.assertEquals(Integer.parseInt(expectedYearTekst) - 100, interval.getStart().getYear());
	}
	
	@Test
	public void createQueryInterval_PatientIdbelowCurrentYearTest() {
		// Arrange
		DateTime currentDate = new DateTime();
		String currentYearTekst = currentDate.getYear() + "";
		String currentYearDecade = currentYearTekst.substring(currentYearTekst.length() - 2, currentYearTekst.length());
		String expectedYearTekst = currentYearTekst.substring(0, currentYearTekst.length() - 2) + currentYearDecade;
		
		// Act
		UnboundableInterval interval = QueryIntervalCreator.deriveIntervalFromPatientDateOfBirth("0101" + currentYearDecade + "1234");
		
		// Assert
		Assert.assertEquals(Integer.parseInt(expectedYearTekst), interval.getStart().getYear());
	}
	
	@Test
	public void startAndStopDerivedFromPatientIdWhenNoStartStopInQuery() {
	    UnboundableInterval interval = QueryIntervalCreator.createQueryInterval(patientId, null, null);
	    
	    Assert.assertEquals(QueryIntervalCreator.deriveIntervalFromPatientDateOfBirth(patientId).getStart(), interval.getStart());
	    Assert.assertNull(interval.getEnd());
	}

	@Test
	public void startAndStopFromQueryUsed() {
	    UnboundableInterval interval = QueryIntervalCreator.createQueryInterval(patientId, start, end);
	    
	    Assert.assertEquals(start, interval.getStart());
	    Assert.assertEquals(end, interval.getEnd());
	}

	@Test
	public void startAndNulledStopFromQueryUsed() {
	    UnboundableInterval interval = QueryIntervalCreator.createQueryInterval(patientId, start, null);
	    
	    Assert.assertEquals(start, interval.getStart());
	    Assert.assertEquals(null, interval.getEnd());
	}

	@Test
	public void nulledStartAndStopFromQueryUsed() {
	    UnboundableInterval interval = QueryIntervalCreator.createQueryInterval(patientId, null, end);
	    Assert.assertEquals(null, interval.getStart());
	    Assert.assertEquals(end, interval.getEnd());
	}
}
