/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsi.dds.projects.sxa.common;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.openehealth.ipf.commons.ihe.xds.core.metadata.Code;

import dk.nsi.dds.projects.sxa.common.configuration.SXAMappedValue;
import dk.nsi.dds.projects.sxa.common.util.ConverterUtil;

public class TestDataHelper {

//	public Code practiceSettingCodeForMicrobiology = new Code("408454008", new LocalizedString("klinisk mikrobiologi"), "2.16.840.1.113883.6.96");
//	public Code practiceSettingCodeForBiochemistry = new Code("394596001", new LocalizedString("klinisk biokemi"), "2.16.840.1.113883.6.96");
//	public Code practiceSettingCodeForPathology = new Code("394915009", new LocalizedString("patologisk anatomi og cytologi"), "2.16.840.1.113883.6.96");
//	public Code practiceSettingCodeNonSxa = new Code("010101010", new LocalizedString("Test"), "2.16.840.1.113883.6.96");

	public SXAMappedValue practiceSettingCodeForMicrobiology = new SXAMappedValue("408454008", "2.16.840.1.113883.6.96", "klinisk mikrobiologi", "Mikrobiologi");
	public SXAMappedValue practiceSettingCodeForBiochemistry = new SXAMappedValue("394596001", "2.16.840.1.113883.6.96", "klinisk biokemi", "KliniskBiokemi");
	public SXAMappedValue practiceSettingCodeForPathology = new SXAMappedValue("394915009", "2.16.840.1.113883.6.96", "patologisk anatomi og cytologi", "Patologi");
	
	public SXAMappedValue practiceSettingCodeNonSxa = new SXAMappedValue("010101010", "2.16.840.1.113883.6.96", "Test", "Test");
	public SXAMappedValue practiceSettingInvalidCodeForPathology = new SXAMappedValue("884915009", "2.16.840.1.113883.6.96", "not the real patologisk anatomi og cytologi", "Patologi");
	
	public static final String DEFAULT_PATIENT_ID = "1212121212";

	public List<SXAMappedValue> getPracticeSettingCodes() {
		return Arrays.asList(practiceSettingCodeForMicrobiology, practiceSettingCodeForBiochemistry, practiceSettingCodeForPathology);
	}
	
	public List<Code> getPracticeSettingCodesAsCodeList() {
		return asCodeList(getPracticeSettingCodes());
	}

	public static List<Code> asCodeList(SXAMappedValue...mappedValues) {
		return asCodeList(Arrays.asList(mappedValues));
	}
	
	public static List<Code> asCodeList(List<SXAMappedValue> mappedValues) {
		return mappedValues.stream().map(e -> ConverterUtil.convert(e)).collect(Collectors.toList());
	}

//	public List<Code> getPracticeSettingCodes() {
//		return Arrays.asList(practiceSettingCodeForMicrobiology, practiceSettingCodeForBiochemistry, practiceSettingCodeForPathology);
//	}
}
