package dk.nsi.dds.projects.sxa.common.configuration;

import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.naming.NamingException;
import javax.naming.spi.NamingManager;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.xml.namespace.QName;

import org.apache.cxf.configuration.jsse.TLSClientParameters;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import org.apache.cxf.transport.http.HTTPConduit;
import org.openehealth.ipf.commons.ihe.ws.WsTransactionConfiguration;
import org.openehealth.ipf.commons.ihe.xds.core.XdsClientFactory;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AssigningAuthority;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Identifiable;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Name;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Person;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.XcnName;
import org.openehealth.ipf.commons.ihe.xds.iti18.Iti18PortType;
import org.openehealth.ipf.commons.ihe.xds.iti43.Iti43PortType;
import org.springframework.beans.factory.annotation.Autowire;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.support.PropertySourcesPlaceholderConfigurer;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.FileSystemResource;
import org.springframework.core.io.Resource;
import org.springframework.mock.jndi.SimpleNamingContextBuilder;

import ca.uhn.hl7v2.model.v25.datatype.XCN;

@Configuration
@ComponentScan(basePackages = "dk.nsi.dds.projects.sxa")
public class IntegrationTestConfiguration {

	public static final String clientPropertiesFileName = "testclient.properties";
	
	@Value("${sxa.iti18.endpoint}")
	private String xdsIti18Endpoint;

	@Value("${sxa.iti43.endpoint}")
	private String xdsIti43Endpoint;

	@Autowired
	protected ApplicationContext appContext;

	private TrustManager[] trustAllCerts = new TrustManager[] { new X509TrustManager() {
		public java.security.cert.X509Certificate[] getAcceptedIssuers() {
			return null;
		}

		public void checkClientTrusted(X509Certificate[] certs, String authType) {
		}

		public void checkServerTrusted(X509Certificate[] certs, String authType) {
		}
	} };

	@PostConstruct
	public void init() {
		Map<String, Iti18PortType> iti18PortTypeBeans = appContext.getBeansOfType(Iti18PortType.class);
		Map<String, Iti43PortType> iti43PortTypeBeans = appContext.getBeansOfType(Iti43PortType.class);
		addInterceptor(iti18PortTypeBeans.values(), iti43PortTypeBeans.values());

		try {
			SimpleNamingContextBuilder builder = new SimpleNamingContextBuilder();
			NamingManager.setInitialContextFactoryBuilder(builder);
		} catch (NamingException e) {
			e.printStackTrace();
		}
	}

	private void addInterceptor(Collection<Iti18PortType> iti18PortTypeBeans, Collection<Iti43PortType> iti43PortTypeBeans) {
		DisableMustUnderstandInterceptor dmui = appContext.getBean(DisableMustUnderstandInterceptor.class);
		AddSingleHttpHeaderInterceptor ashhi = new AddSingleHttpHeaderInterceptor("X-Labsvar-Credential", "00000000-0000-0000-0000-000000000000");
		
		for (Iti18PortType iti18PortType : iti18PortTypeBeans) {
			ClientProxy.getClient(iti18PortType).getOutInterceptors().add(dmui);
			ClientProxy.getClient(iti18PortType).getOutInterceptors().add(ashhi);
		}
		for (Iti43PortType iti43PortType : iti43PortTypeBeans) {
			ClientProxy.getClient(iti43PortType).getOutInterceptors().add(dmui);
			ClientProxy.getClient(iti43PortType).getOutInterceptors().add(ashhi);
		}
	}

	@Bean(autowire = Autowire.BY_NAME, name = "iti18PortType")
	public Iti18PortType getDocumentRegistryServiceIti18() {
		return getDocumentRegistryService(xdsIti18Endpoint);
	}

	@Bean(autowire = Autowire.BY_NAME, name = "iti43PortType")
	public Iti43PortType getDocumentRepositoryServiceIti43() {
		return getDocumentRepositoryService(xdsIti43Endpoint);
	}


	private Iti43PortType getDocumentRepositoryService(final String endpoint) {
		QName service = new QName("urn:ihe:iti:xds-b:2007", "DocumentRepository_Service", "ihe");
		QName binding = new QName("urn:ihe:iti:xds-b:2007", "DocumentRepository_Binding_Soap12", "ihe");

		String wsdl = "wsdl/iti43.wsdl";

		return getDocumentService(service, binding, wsdl, endpoint, Iti43PortType.class);
	}

	private Iti18PortType getDocumentRegistryService(final String endpoint) {
		QName service = new QName("urn:ihe:iti:xds-b:2007", "DocumentRegistry_Service", "ihe");
		QName binding = new QName("urn:ihe:iti:xds-b:2007", "DocumentRegistry_Binding_Soap12", "ihe");

		String wsdl = "wsdl/iti18.wsdl";

		return getDocumentService(service, binding, wsdl, endpoint, Iti18PortType.class);
	}

	private <E> E getDocumentService(final QName service, final QName binding, final String wsdl, final String endpoint, Class<E> c) {
		XdsClientFactory xdsClientFactory = generateXdsClientFactory(service, binding, wsdl, endpoint, c);
		
		@SuppressWarnings("unchecked")
		E client = (E) xdsClientFactory.getClient();
		initProxy(client);
		return client;
	}

	@Bean
	public DisableMustUnderstandInterceptor mustUnderstandInterceptor() {
		return new DisableMustUnderstandInterceptor();
	}

	@Bean
	public Person getTestPerson() {
		Identifiable id = new Identifiable("2512484916", new AssigningAuthority("1.2.208.176.1.2"));
		Name<XCN> name = new XcnName("surName", "givenName", "middleName", "suffix", "prefix", "HighDegree");
		return new Person(id, name);
	}

	private XdsClientFactory generateXdsClientFactory(QName service, QName binding, String wsdl, String url, Class<?> clazz) {
		final WsTransactionConfiguration WS_CONFIG = new WsTransactionConfiguration(service, clazz, binding, false, wsdl, true, false, false, false);
		return new XdsClientFactory(WS_CONFIG, url, null, null, null);
	}

	private void initProxy(Object o) {
		Client proxy = ClientProxy.getClient(o);
		proxy.getOutInterceptors().add(new LoggingOutInterceptor());
		proxy.getInInterceptors().add(new LoggingInInterceptor());
		HTTPConduit conduit = (HTTPConduit) proxy.getConduit();
		TLSClientParameters tcp = new TLSClientParameters();
		tcp.setTrustManagers(trustAllCerts);
		conduit.setTlsClientParameters(tcp);
	}
	
	private static List<Resource> getResources(){
		final List<Resource> resources = new ArrayList<Resource>();
		
		resources.add(new ClassPathResource(clientPropertiesFileName));
		
		if(System.getProperty(clientPropertiesFileName) != null) {
			resources.add(new FileSystemResource(System.getProperty(clientPropertiesFileName)))  ;
		}
		return resources;
	}

	@Bean
	public static PropertySourcesPlaceholderConfigurer properties() {
		final PropertySourcesPlaceholderConfigurer ppc = new PropertySourcesPlaceholderConfigurer();
		ppc.setLocations(getResources().toArray(new Resource[0]));
		ppc.setIgnoreUnresolvablePlaceholders(false);
		ppc.setIgnoreResourceNotFound(true);
		return ppc;
	}
}
