/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common.services.security;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;

import org.apache.commons.io.IOUtils;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicHeader;

@SuppressWarnings("deprecation")
public class RequestHelper {
	

	private static final Charset UTF_8 = Charset.forName("utf-8");

	public static String sendRequest(String url, String soapAction, String postBody) throws IOException {
		
		try(DefaultHttpClient defaultHttpClient = new DefaultHttpClient()){
			defaultHttpClient.getParams().setParameter("http.connection.timeout", Integer.valueOf(30000));
			HttpPost httpPost = new HttpPost(url);
			httpPost.addHeader(new BasicHeader("Content-Type", "text/xml; charset=utf-8"));
			httpPost.addHeader(new BasicHeader("SOAPAction", "\"" + soapAction + "\""));
	
			httpPost.setEntity(new StringEntity(postBody, UTF_8));
			HttpResponse response = defaultHttpClient.execute(httpPost);
			String result = "";
			int statusCode = response.getStatusLine().getStatusCode();
			if ((statusCode == 200) || (statusCode == 500)) {
				InputStream is = null;
				try {
					is = response.getEntity().getContent();
					result = IOUtils.toString(is, UTF_8.toString());
				} finally {
					if (is != null)
						is.close();
				}
			} else {
				if (statusCode == 202) {
					return "";
				}
				result = response.getStatusLine().getReasonPhrase();
				throw new IOException("HTTP POST failed (" + statusCode + "): " + result);
			}
			return result;
		}
	}
}
