/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common.testutil;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.builder.ReflectionToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocument;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.FindDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.responses.RetrievedDocument;

public class DocumentSharingTestUtil {
	public String prettyPrint(RetrieveDocument meta) {
		return prettyPrintInternal(meta);
	}

	public String prettyPrint(FindDocumentsQuery query) {
		return prettyPrintInternal(query);
	}

//	public String prettyPrint(RetrieveDocument queryResult) {
//		return prettyPrintInternal(queryResult);
//	}

	public String prettyPrint(List<?> fetchList) {
		return prettyPrintInternal(fetchList);
	}

	public String prettyPrint(RetrievedDocument docRetrieved) {
		return prettyPrintInternal(new RetrievedDocumentWrapper(docRetrieved));
	}

//	public String prettyPrint(List<RetrievedDocument> retrievedDocuments) {
//		return prettyPrintInternal(retrievedDocuments);
//	}

	public List<RetrieveDocument> filter(List<RetrieveDocument> docToRetrieve, String documentId) {
		List<RetrieveDocument> filtered = new ArrayList<>();
		for (RetrieveDocument entry : docToRetrieve) {
			if (entry.getDocumentUniqueId().equals(documentId)) {
				filtered.add(entry);
				break;
			}
		}
		return filtered;
	}

	public List<RetrieveDocument> remove(List<RetrieveDocument> docToRetrieve, String repositoryUniqueId) {
		List<RetrieveDocument> filtered = new ArrayList<>();
		for (RetrieveDocument entry : docToRetrieve) {
			if (!entry.getRepositoryUniqueId().equals(repositoryUniqueId)) {
				filtered.add(entry);
			}
		}
		return filtered;
	}

	private String prettyPrintInternal(Object obj) {
		return ReflectionToStringBuilder.toString(obj, ToStringStyle.MULTI_LINE_STYLE);
	}

	private class RetrievedDocumentWrapper {
		@SuppressWarnings("unused")
		private RetrievedDocument retrievedDoc;

		public RetrievedDocumentWrapper(RetrievedDocument retrievedDoc) {
			this.retrievedDoc = retrievedDoc;
		}
	}
}
