/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common.xds.impl;

import static org.junit.Assert.fail;

import org.junit.Assert;
import org.junit.Test;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AssigningAuthority;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Identifiable;

import dk.nsi.dds.projects.sxa.common.xds.UnexpectedDataException;

public class ValueEncoderDecoderOfPatientIdTest {
	private static final Identifiable VALID_DECODED_PATIENT_ID = new Identifiable("id", new AssigningAuthority("uid", "type"));
	private static final String VALID_ENCODED_PATIENT_ID = "id^^^&uid&type";

	@Test
	public void testDecodeNull() {
		String encodedOrganization = null;
		try {
			ValueEncoderDecoder.decodePatientId(encodedOrganization);
			fail("Expected to throw IllegalArgumentException");
		} catch (IllegalArgumentException e) {
			Assert.assertEquals("encodedPatientId missing", e.getMessage());
		}
	}

	@Test
	public void testDecodeValidEncoded() {
		String encodedPatientId = VALID_ENCODED_PATIENT_ID;
		Identifiable patientId = ValueEncoderDecoder.decodePatientId(encodedPatientId);
		Assert.assertEquals("Identifiable[id=id,assigningAuthority=AssigningAuthority[universalId=uid,universalIdType=type]]", patientId.toString());

		Assert.assertEquals("id", patientId.getId());
		Assert.assertNotNull(patientId.getAssigningAuthority());
		Assert.assertEquals("uid", patientId.getAssigningAuthority().getUniversalId());
		Assert.assertEquals("type", patientId.getAssigningAuthority().getUniversalIdType());
	}

	@Test
	public void testEncodeNull() {
		try {
			Identifiable patientId = null;
			String encodePatientId = ValueEncoderDecoder.encodePatientId(patientId);
			Assert.assertNull(encodePatientId);
		} catch (UnexpectedDataException e) {
			fail("Not expected to throw exception");
		}
	}

	@Test
	public void testEncodeValid() throws UnexpectedDataException {
		Identifiable patientId = VALID_DECODED_PATIENT_ID;
		String encodedPatientId = ValueEncoderDecoder.encodePatientId(patientId);
		Assert.assertEquals(VALID_ENCODED_PATIENT_ID, encodedPatientId);
	}
}
