/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (SDS), Denmark
 * (http://sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (SDS), Denmark (http://sundhedsdatastyrelsen.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.common.xds.impl;

import static org.junit.Assert.fail;

import org.junit.Assert;
import org.junit.Test;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AssigningAuthority;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Identifiable;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Name;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Person;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.XcnName;

import ca.uhn.hl7v2.model.v25.datatype.XCN;
import dk.nsi.dds.projects.sxa.common.xds.UnexpectedDataException;

public class ValueEncoderDecoderOfPersonTest {
	private static final Identifiable VALID_DECODED_PERSON_ID = new Identifiable("id", new AssigningAuthority("uid", "type"));
	private static final Name<XCN> VALID_DECODED_PERSON_NAME = new XcnName("family", "given", "middle", "suffix", "prefix", "degree");
	private static final Person VALID_DECODED_PERSON = new Person(VALID_DECODED_PERSON_ID, VALID_DECODED_PERSON_NAME);
	private static final String VALID_ENCODED_PERSON = "id^family^given^middle^suffix^prefix^degree^^&uid&type";

	@Test
	public void testDecodeNull() {
		String encodedPerson = null;
		try {
			ValueEncoderDecoder.decodePerson(encodedPerson);
			fail("Expected to throw IllegalArgumentException");
		} catch (IllegalArgumentException e) {
			Assert.assertEquals("encodedPerson missing", e.getMessage());
		}
	}

	@Test
	public void testDecodeValidEncoded() {
		String encodedPerson = VALID_ENCODED_PERSON;
		Person person = ValueEncoderDecoder.decodePerson(encodedPerson);
		Assert.assertEquals(
				"Person[id=Identifiable[id=id,assigningAuthority=AssigningAuthority[universalId=uid,universalIdType=type]],name=XcnName[family=family,given=given,secondAndFurtherGiven=middle,suffix=suffix,prefix=prefix,degree=degree]]",
				person.toString());

		Assert.assertNotNull(person.getId());
		Assert.assertEquals("id", person.getId().getId());
		Assert.assertNotNull(person.getId().getAssigningAuthority());
		Assert.assertEquals("uid", person.getId().getAssigningAuthority().getUniversalId());
		Assert.assertEquals("type", person.getId().getAssigningAuthority().getUniversalIdType());
		Assert.assertNotNull(person.getName());
		Assert.assertEquals("family", person.getName().getFamilyName());
		Assert.assertEquals("given", person.getName().getGivenName());
		Assert.assertEquals("middle", person.getName().getSecondAndFurtherGivenNames());
		Assert.assertEquals("suffix", person.getName().getSuffix());
		Assert.assertEquals("prefix", person.getName().getPrefix());
		Assert.assertEquals("degree", person.getName().getDegree());
	}

	@Test
	public void testEncodeNull() {
		Person person = null;
		try {
			ValueEncoderDecoder.encodePerson(person);
			fail("Expected to throw UnexpectedDataException");
		} catch (UnexpectedDataException e) {
			Assert.assertEquals("Values missing for person", e.getMessage());
		}
	}

	@Test
	public void testEncodeValid() throws UnexpectedDataException {
		Person person = VALID_DECODED_PERSON;
		String encodedPerson = ValueEncoderDecoder.encodePerson(person);
		Assert.assertEquals(VALID_ENCODED_PERSON, encodedPerson);
	}
}
