/**
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark
 * (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.nsi.dds.projects.sxa.versioninfo;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Properties;

import javax.servlet.http.HttpServletResponse;

import org.codehaus.plexus.util.ReflectionUtils;
import org.junit.Test;
import org.mockito.Mockito;

import org.junit.Assert;

public class VersionInfoServletTest {
	private static final String TEST_PROPERTIES_FILE = "/testpom.properties";
	
	@Test
	public void validPomPropertiesFileTest() throws Exception {
		VersionInfoServlet servlet = new VersionInfoServlet();
		ReflectionUtils.setVariableValueInObject(servlet, "propertiesFile", TEST_PROPERTIES_FILE);
		String expectedResult = "Version: " + getVersionFromResource();
		String actualResult = callVersionInfoServletUsingResponseMock(servlet);
		Assert.assertTrue("[" + actualResult + "] doesn't match [" + expectedResult + "]",
				actualResult.contains(expectedResult));
	}
	
	@Test
	public void missingPomPropertiesFileTest() throws Exception {
		VersionInfoServlet servlet = new VersionInfoServlet();
		ReflectionUtils.setVariableValueInObject(servlet, "propertiesFile", "non-existing-file.xml");
		String expectedResult = "Version: null";
		String actualResult = callVersionInfoServletUsingResponseMock(servlet);
		Assert.assertTrue("[" + actualResult + "] doesn't match [" + expectedResult + "]",
				actualResult.contains(expectedResult));
	}
	
	private String callVersionInfoServletUsingResponseMock(VersionInfoServlet servlet) throws Exception {
		StringWriter stringWriter = new StringWriter();
		HttpServletResponse responseMock = Mockito.mock(HttpServletResponse.class);
		Mockito.when(responseMock.getWriter()).thenReturn(new PrintWriter(stringWriter));
		servlet.doGet(null, responseMock);
		return stringWriter.toString();
	}
	
    private String getVersionFromResource() throws Exception {
	    InputStream resAsStream = this.getClass().getResourceAsStream(TEST_PROPERTIES_FILE);
	    if (resAsStream == null) {
	    	throw new FileNotFoundException(TEST_PROPERTIES_FILE + " is missing");
	    }
	    String result = null;
	    try {
			Properties props = new Properties(); 
			props.load(resAsStream);
			result = props.getProperty("version");
	    } catch (IOException e) {
	    	// Ignore
	    }
	    finally {
			if (resAsStream != null) {
			    try {
			    	resAsStream.close();
			    }
			    catch (IOException e) {
			    	; // ignore
			    }
			}
	    }
	    return result;
    }
}
